@@ -6,6 +6,7 @@ $directive_type_count = 20;
 if ($module_name ne 'htaccess') {
 	do '../web-lib.pl';
 	&init_config();
+	require '../ui-lib.pl';
 	%access = &get_module_acl();
 	@access_types = $access{'types'} eq '*' ? (0 .. $directive_type_count)
 					: split(/\s+/, $access{'types'});
@@ -14,16 +15,37 @@ else {
 	@access_types = (0 .. $directive_type_count);
 	}
 map { $access_types{$_}++ } @access_types;
+$site_file = ($config{'webmin_apache'} || $module_config_directory)."/site";
+
+if (!-r $site_file && $module_name ne 'htaccess') {
+	# Need to build list of supported modules
+	local $httpd = &find_httpd();
+	local ($ver, $mods) = &httpd_info($httpd);
+	if ($ver) {
+		local @mods = map { "$_/$ver" } &configurable_modules();
+		local @st = stat($httpd);
+		local %site = ( 'size' => $st[7],
+				'modules' => join(' ', @mods) );
+		&lock_file($site_file);
+		&write_file($site_file, \%site);
+		chmod(0644, $site_file);
+		&unlock_file($site_file);
+		}
+	}
 
 # Read the site-specific setup file, then require in all the module-specific
 # .pl files
-local $mcd = $config{'webmin_apache'} || $module_config_directory;
-if (&read_file("$mcd/site", \%site)) {
+if (&read_file($site_file, \%site)) {
 	local($m, $f, $d);
 	$httpd_size = $site{'size'};
 	foreach $m (split(/\s+/, $site{'modules'})) {
 		if ($m =~ /(\S+)\/(\S+)/) { $httpd_modules{$1} = $2; }
 		}
+	foreach $m (keys %httpd_modules) {
+		if (!-r "$module_root_directory/$m.pl") {
+			delete($httpd_modules{$m});
+			}
+		}
 	foreach $f (split(/\s+/, $site{'htaccess'})) {
 		if (-r $f) { push(@htaccess_files, $f); }
 		}
@@ -191,7 +213,7 @@ return join(' ', @rv);
 # Returns the values of directives matching some name
 sub find_directive
 {
-local(@rv, $i, @vals, $dref);
+local (@vals, $ref);
 foreach $ref (@{$_[1]}) {
 	if (lc($ref->{'name'}) eq lc($_[0])) {
 		push(@vals, $_[2] ? $ref->{'words'}->[0] : $ref->{'value'});
@@ -204,7 +226,7 @@ return wantarray ? @vals : !@vals ? undef : $vals[$#vals];
 # Returns references to directives maching some name
 sub find_directive_struct
 {
-local(@rv, $i, @vals);
+local (@vals, $ref);
 foreach $ref (@{$_[1]}) {
 	if (lc($ref->{'name'}) eq lc($_[0])) {
 		push(@vals, $ref);
@@ -244,14 +266,18 @@ $lnum = 0;
 $res = &find_directive("ResourceConfig", \@get_config_cache);
 if (!$res) { $res = $config{'srm_conf'}; }
 if (!$res) { $res = "$config{'httpd_dir'}/conf/srm.conf"; }
-if (!-r $res) { $res = "$config{'httpd_dir'}/etc/srm.conf"; }
+if (!-r &translate_filename($res)) {
+	$res = "$config{'httpd_dir'}/etc/srm.conf";
+	}
 push(@get_config_cache, &get_config_file($res));
 
 $lnum = 0;
 $acc = &find_directive("AccessConfig", \@get_config_cache);
 if (!$acc) { $acc = $config{'access_conf'}; }
 if (!$acc) { $acc = "$config{'httpd_dir'}/conf/access.conf"; }
-if (!-r $acc) { $acc = "$config{'httpd_dir'}/etc/access.conf"; }
+if (!-r &translate_filename($acc)) {
+	$acc = "$config{'httpd_dir'}/etc/access.conf";
+	}
 push(@get_config_cache, &get_config_file($acc));
 
 # Read extra config files in VirtualHost sections
@@ -288,7 +314,7 @@ if (opendir(DIR, $_[0])) {
 else {
 	# Just a normal config file
 	local $lnum = 0;
-	open(CONF, $_[0]);
+	&open_readfile(CONF, $_[0]);
 	@rv = &parse_config_file(CONF, $lnum, $_[0]);
 	close(CONF);
 	}
@@ -324,7 +350,7 @@ return wantarray ? ($c, $v) : $c;
 sub get_htaccess_config
 {
 local($lnum, @conf);
-open(HTACCESS, $_[0]);
+&open_readfile(HTACCESS, $_[0]);
 @conf = &parse_config_file(HTACCESS, $lnum, $_[0]);
 close(HTACCESS);
 return \@conf;
@@ -770,7 +796,7 @@ else { return "$dfm <tt>".&html_escape($_[0]->{'words'}->[0])."</tt>"; }
 sub list_user_file
 {
 local($_);
-open(USERS, $_[0]);
+&open_readfile(USERS, $_[0]);
 while(<USERS>) {
 	/^(\S+):(\S+)/;
 	push(@{$_[1]}, $1); $_[2]->{$1} = $2;
@@ -791,27 +817,21 @@ foreach $m (sort { $a cmp $b } (keys %httpd_modules)) {
 		if ($e->{$_[0]}) { $etype{$e->{'type'}}++; }
 		}
         }
-print "<table width=100% cellpadding=5>\n";
+local (@titles, @links, @icons);
 for($i=0; $text{"type_$i"}; $i++) {
 	if ($etype{$i} && $access_types{$i}) {
-		if ($c%5 == 0) { print "<tr>\n"; }
-		print "<td valign=top align=center width=20%>\n";
-		&generate_icon("images/type_icon_$i.gif", $text{"type_$i"},
-			       $_[1]."type=$i");
-		print "</td>\n";
-		if ($c%5 == 4) { print "<tr>\n"; }
-		$c++;
+		push(@links, $_[1]."type=$i");
+		push(@titles, $text{"type_$i"});
+		push(@icons, "images/type_icon_$i.gif");
 		}
 	}
 for($i=2; $i<@_; $i++) {
-	if ($c%5 == 0) { print "<tr>\n"; }
-	print "<td valign=top align=center width=20%>\n";
-	&generate_icon($_[$i]->{'icon'}, $_[$i]->{'name'}, $_[$i]->{'link'});
-	print "</td>\n";
-	if ($c%5 == 4) { print "<tr>\n"; }
-	$c++;
+	push(@links, $_[$i]->{'link'});
+	push(@titles, $_[$i]->{'name'});
+	push(@icons, $_[$i]->{'icon'});
 	}
-print "</table><p>\n";
+&icons_table(\@links, \@titles, \@icons, 5);
+print "<p>\n";
 }
 
 # restart_button()
@@ -821,8 +841,7 @@ sub restart_button
 local $rv;
 local $pidfile = &get_pid_file();
 $args = "redir=".&urlize(&this_url());
-if (open(PID, $pidfile) && <PID> =~ /(\d+)/ && kill(0, $1)) {
-	close(PID);
+if (&check_pid_file($pidfile)) {
 	if ($access{'apply'}) {
 		$rv .= "<a href=\"restart.cgi?$args\">$text{'apache_apply'}</a><br>\n";
 		}
@@ -865,7 +884,7 @@ return @rv;
 sub httpd_info
 {
 local(@mods, $verstr, $ver, $minor);
-$verstr = `$_[0] -v 2>&1`;
+$verstr = &backquote_command("$_[0] -v 2>&1");
 if ($config{'httpd_version'}) {
 	$config{'httpd_version'} =~ /(\d+)\.([\d\.]+)/;
 	$ver = $1; $minor = $2; $minor =~ s/\.//g; $ver .= ".$minor";
@@ -886,7 +905,7 @@ elsif ($verstr =~ /Red\s*Hat\s+Secure\/3\.0/i) {
 	# redhat secure server 3.0
 	$ver = 1.39;
 	}
-elsif (`rpm -q apache 2>&1` =~ /^apache-(\d+)\.([\d\.]+)/) {
+elsif (&backquote_command("rpm -q apache 2>&1") =~ /^apache-(\d+)\.([\d\.]+)/) {
 	# got version from the RPM
 	$ver = $1; $minor = $2; $minor =~ s/\.//g; $ver .= ".$minor";
 	}
@@ -903,7 +922,7 @@ if ($ver < 1.2) {
 else {
 	# ask apache for the module list
 	@mods = ("core");
-	open(APACHE, "$_[0] -l |");
+	&open_execute_command(APACHE, "$_[0] -l", 1);
 	while(<APACHE>) {
 		if (/(\S+)\.c/) { push(@mods, $1); }
 		}
@@ -1037,7 +1056,8 @@ sub test_config
 if ($httpd_modules{'core'} >= 1.301) {
 	# Test the configuration with the available command
 	local $cmd;
-	if ($config{'test_apachectl'} && -x $config{'apachectl_path'}) {
+	if ($config{'test_apachectl'} &&
+	    -x &translate_filename($config{'apachectl_path'})) {
 		# Test with apachectl
 		$cmd = "$config{'apachectl_path'} configtest";
 		}
@@ -1052,7 +1072,7 @@ if ($httpd_modules{'core'} >= 1.301) {
 			$cmd .= " -D$d";
 			}
 		}
-	local $out = `$cmd 2>&1`;
+	local $out = &backquote_command("$cmd 2>&1");
 	if ($out && $out !~ /syntax\s+ok/i) {
 		return $out;
 		}
@@ -1071,7 +1091,7 @@ if ($config{'test_always'} || $access{'test_always'}) {
 	local $/ = undef;
 	local $f;
 	foreach $f (@files) {
-		if (open(BEFORE, $f)) {
+		if (&open_readfile(BEFORE, $f)) {
 			$before_changing{$f} = <BEFORE>;
 			close(BEFORE);
 			}
@@ -1088,28 +1108,39 @@ if ($config{'test_always'} || $access{'test_always'}) {
 	local $err = &test_config();
 	if ($err) {
 		# Something failed .. revert all files
-		local $f;
-		foreach $f (keys %before_changing) {
-			open(AFTER, ">$f");
-			print AFTER $before_changing{$f};
-			close(AFTER);
-			}
+		&rollback_apache_config();
 		&error(&text('eafter', "<pre>$err</pre>"));
 		}
 	}
 }
 
+# rollback_apache_config()
+# Copy back all config files from their originals
+sub rollback_apache_config
+{
+local $f;
+foreach $f (keys %before_changing) {
+	&open_tempfile(AFTER, ">$f");
+	&print_tempfile(AFTER, $before_changing{$f});
+	&close_tempfile(AFTER);
+	}
+}
+
 # find_httpd_conf()
 # Returns the path to the http.conf file, and the last place looked
+# (without any translation).
 sub find_httpd_conf
 {
 local $conf = $config{'httpd_conf'};
-return ( -r $conf ? $conf : undef, $conf ) if ($conf);
+return ( -r &translate_filename($conf) ? $conf : undef, $conf ) if ($conf);
 $conf = "$config{'httpd_dir'}/conf/httpd.conf";
-$conf = "$config{'httpd_dir'}/conf/httpd2.conf" if (!-r $conf);
-$conf = "$config{'httpd_dir'}/etc/httpd.conf" if (!-r $conf);
-$conf = "$config{'httpd_dir'}/etc/httpd2.conf" if (!-r $conf);
-$conf = undef if (!-r $conf);
+$conf = "$config{'httpd_dir'}/conf/httpd2.conf"
+	if (!-r &translate_filename($conf));
+$conf = "$config{'httpd_dir'}/etc/httpd.conf"
+	if (!-r &translate_filename($conf));
+$conf = "$config{'httpd_dir'}/etc/httpd2.conf"
+	if (!-r &translate_filename($conf));
+$conf = undef if (!-r &translate_filename($conf));
 return ( $conf, "$config{'httpd_dir'}/conf/httpd.conf" );
 }
 
@@ -1117,15 +1148,17 @@ return ( $conf, "$config{'httpd_dir'}/conf/httpd.conf" );
 # Returns the path to the httpd executable, by appending '2' if necessary
 sub find_httpd
 {
-return $config{'httpd_path'} if (-x $config{'httpd_path'} &&
-				 !-d $config{'httpd_path'});
-return $config{'httpd_path'}.'2' if (-x $config{'httpd_path'}.'2' &&
-				     !-d $config{'httpd_path'}.'2');
+return $config{'httpd_path'}
+	if (-x &translate_filename($config{'httpd_path'}) &&
+	    !-d &translate_filename($config{'httpd_path'}));
+return $config{'httpd_path'}.'2'
+	if (-x &translate_filename($config{'httpd_path'}.'2') &&
+	    !-d &translate_filename($config{'httpd_path'}.'2'));
 return undef;
 }
 
 # get_pid_file()
-# Returns the path to the PID file
+# Returns the path to the PID file (without any translation)
 sub get_pid_file
 {
 return $config{'pid_file'} if ($config{'pid_file'});
@@ -1136,5 +1169,184 @@ local $pidfile = $pidfilestr ? $pidfilestr->{'words'}->[0]
 return &server_root($pidfile, $conf);
 }
 
+# restart_apache()
+# Re-starts Apache, and returns undef on success or an error message on failure
+sub restart_apache
+{
+local $conf = &get_config();
+local $pidfile = &get_pid_file();
+if ($config{'apply_cmd'}) {
+	# use the configured start command
+	local $out = &backquote_logged("($config{'apply_cmd'}) 2>&1");
+	if ($?) {
+		return "<pre>$out</pre>";
+		}
+	}
+elsif (-x &translate_filename($config{'apachectl_path'})) {
+	# use apachectl to restart
+	if ($httpd_modules{'core'} >= 2) {
+		# Do a graceful restart
+		local $out = &backquote_logged("$config{'apachectl_path'} graceful 2>&1");
+		if ($?) {
+			return "<pre>$out</pre>";
+			}
+		}
+	else {
+		local $out = &backquote_logged("$config{'apachectl_path'} restart 2>&1");
+		if ($out !~ /httpd restarted/) {
+			return "<pre>$out</pre>";
+			}
+		}
+	}
+else {
+	# send SIGHUP directly
+	&open_readfile(PID, $pidfile) || return &text('restart_epid', $pidfile);
+	<PID> =~ /(\d+)/ || return &text('restart_epid2', $pidfile);
+	close(PID);
+	&kill_logged('HUP', $1) || return &text('restart_esig', $1);
+	}
+return undef;
+}
+
+# stop_apache()
+# Attempts to stop the running Apache process, and returns undef on success or
+# an error message on failure
+sub stop_apache
+{
+local $out;
+if ($config{'stop_cmd'}) {
+	# use the configured stop command
+	$out = &backquote_logged("($config{'stop_cmd'}) 2>&1");
+	if ($?) {
+		return "<pre>$out</pre>";
+		}
+	}
+elsif (-x $config{'apachectl_path'}) {
+	# use the apachectl program
+	$out = &backquote_logged("($config{'apachectl_path'} stop) 2>&1");
+	if ($httpd_modules{'core'} >= 2 ? $? : $out !~ /httpd stopped/) {
+		return "<pre>$out</pre>";
+		}
+	}
+else {
+	# kill the process
+	$pidfile = &get_pid_file();
+	open(PID, $pidfile) || return &text('stop_epid', $pidfile);
+	<PID> =~ /(\d+)/ || return &text('stop_epid2', $pidfile);
+	close(PID);
+	&kill_logged('TERM', $1) || return &text('stop_esig', $1);
+	}
+return undef;
+}
+
+# start_apache()
+# Attempts to start Apache, and returns undef on success or an error message
+# upon failure.
+sub start_apache
+{
+local $out;
+&clean_environment();
+if ($config{'start_cmd'}) {
+	# use the configured start command
+	if ($config{'stop_cmd'}) {
+		# execute the stop command to clear lock files
+		&system_logged("($config{'stop_cmd'}) >/dev/null 2>&1");
+		}
+	$out = &backquote_logged("($config{'start_cmd'}) 2>&1");
+	&reset_environment();
+	if ($?) {
+		return "<pre>$out</pre>";
+		}
+	}
+elsif (-x $config{'apachectl_path'}) {
+	# use the apachectl program
+	$out = &backquote_logged("($config{'apachectl_path'} start) 2>&1");
+	&reset_environment();
+	if ($out =~ /\S/ && $out !~ /httpd started/) {
+		return "<pre>$out</pre>";
+		}
+	}
+else {
+	# start manually
+	local $httpd = &find_httpd();
+	local $cmd = "$httpd -d $config{'httpd_dir'}";
+	if ($config{'httpd_conf'}) {
+		$cmd .= " -f $config{'httpd_conf'}";
+		}
+	foreach $d (keys %httpd_defines) {
+		$cmd .= " -D$d";
+		}
+	local $temp = &transname();
+	local $rv = &system_logged("( $cmd ) >$temp 2>&1 </dev/null");
+	$out = `cat $temp`;
+	unlink($temp);
+	&reset_environment();
+	if ($out =~ /\S/ && $out !~ /httpd started/) {
+		return "<pre>$cmd :\n$out</pre>";
+		}
+	}
+
+# check if startup was successful. Later apache version return no
+# error code, but instead fail to start and write the reason to
+# the error log file!
+sleep(3);
+local $conf = &get_config();
+local $pidfile = &get_pid_file();
+if (!&check_pid_file($pidfile)) {
+	# Not running..  find out why
+	local $errorlogstr = &find_directive_struct("ErrorLog", $conf);
+	local $errorlog = $errorlogstr ? $errorlogstr->{'words'}->[0]
+				       : "logs/error_log";
+	if ($out =~ /\S/) {
+		return "$text{'start_eafter'} : <pre>$out</pre>";
+		}
+	elsif ($errorlog eq 'syslog' || $errorlog =~ /^\|/) {
+		return $text{'start_eunknown'};
+		}
+	else {
+		$errorlog = &server_root($errorlog, $conf);
+		$out = `tail -5 $errorlog`;
+		return "$text{'start_eafter'} : <pre>$out</pre>";
+		}
+	}
+return undef;
+}
+
+# configurable_modules()
+# Returns a list of Apaches that are compiled in or dynamically loaded, and
+# supported by Webmin.
+sub configurable_modules
+{
+local ($ver, $mods) = &httpd_info(&find_httpd());
+local @rv;
+local $m;
+
+# add compiled-in modules
+foreach $m (@$mods) {
+	if (-r "$module_root_directory/$m.pl") {
+		push(@rv, $m);
+		}
+	}
+
+# add dynamically loaded modules
+local $conf = &get_config();
+foreach $l (&find_directive_struct("LoadModule", $conf)) {
+	if ($l->{'words'}->[1] =~ /(mod_\S+)\.(so|dll)/ &&
+	    -r "$module_root_directory/$1.pl") {
+		push(@rv, $1);
+		}
+	elsif ($l->{'words'}->[1] =~ /libssl\.so/ &&
+	       -r "$module_root_directory/mod_apachessl.pl") {
+		push(@rv, "mod_apachessl");
+		}
+	elsif ($l->{'words'}->[1] =~ /lib([^\/\s]+)\.(so|dll)/ &&
+	       -r "$module_root_directory/mod_$1.pl") {
+		push(@rv, "mod_$1");
+		}
+	}
+
+return &unique(@rv);
+}
+
 1;
 
