@@ -340,7 +340,7 @@ for($i=0; $i<@oldv || $i<@newv; $i++) {
 	if ($i >= @oldv && $_[4]) {
 		# a new directive is being added.. put it at the start of
 		# the parent
-		$lref = &read_file_lines($_[0]->{'file'});
+		$lref = $_[0]->{'file'} ? &read_file_lines($_[0]->{'file'}) : [ ];
 		@nl = &directive_lines($newv[$i], $_[3]);
 		$nline = $_[0]->{'fline'}+1;
 		splice(@$lref, $nline, 0, @nl);
@@ -355,7 +355,7 @@ for($i=0; $i<@oldv || $i<@newv; $i++) {
 	elsif ($i >= @oldv) {
 		# a new directive is being added.. put it at the end of
 		# the parent
-		$lref = &read_file_lines($_[0]->{'file'});
+		$lref = $_[0]->{'file'} ? &read_file_lines($_[0]->{'file'}) : [ ];
 		@nl = &directive_lines($newv[$i], $_[3]);
 		splice(@$lref, $_[0]->{'eline'}, 0, @nl);
 		&renumber(&get_config(), $_[0]->{'eline'},
@@ -367,7 +367,7 @@ for($i=0; $i<@oldv || $i<@newv; $i++) {
 		}
 	elsif ($i >= @newv) {
 		# a directive was deleted
-		$lref = &read_file_lines($oldv[$i]->{'file'});
+		$lref = $oldv[$i]->{'file'} ? &read_file_lines($oldv[$i]->{'file'}) : [ ];
 		$ol = $oldv[$i]->{'eline'} - $oldv[$i]->{'line'} + 1;
 		splice(@$lref, $oldv[$i]->{'line'}, $ol);
 		&renumber(&get_config(), $oldv[$i]->{'eline'},
@@ -377,7 +377,7 @@ for($i=0; $i<@oldv || $i<@newv; $i++) {
 		}
 	else {
 		# updating some directive
-		$lref = &read_file_lines($oldv[$i]->{'file'});
+		$lref = $oldv[$i]->{'file'} ? &read_file_lines($oldv[$i]->{'file'}) : [ ];
 		@nl = &directive_lines($newv[$i], $_[3]);
 		$ol = $oldv[$i]->{'eline'} - $oldv[$i]->{'line'} + 1;
 		splice(@$lref, $oldv[$i]->{'line'}, $ol, @nl);
@@ -480,7 +480,10 @@ sub group_name
 {
 local @opts = &find("option", $_[1]->{'members'});
 local ($dn) = grep { $_->{'values'}->[0] eq 'domain-name' } @opts;
-return  $config{'group_name'} && $dn ? &text('index_gdom',$dn->{'values'}->[1]):
+return  $config{'group_name'} == 1 && $dn ?
+		&text('index_gdom',$dn->{'values'}->[1]) :
+	$config{'group_name'} == 2 && $_[1]->{'comment'} ?
+		$_[1]->{'comment'} :
 	$_[1]->{'values'}->[0] ? $_[1]->{'values'}->[0] :
         $_[0] == 0 ? $text{'index_nomemb'} :
 	$_[0] == 1 ? $text{'index_1memb'} :
@@ -773,6 +776,7 @@ sub get_dhcpd_version
 {
 local $out = `$config{'dhcpd_path'} -v 2>&1`;
 ${$_[0]} = $out;
+return $config{'version'} if ($config{'version'});
 return undef if ($out !~ /DHCP/ || $out =~ /V1/);
 return $out =~ /\sV([0-9\.]+)/ ? $1 :
        $out =~ /-T/ ? 3 : 2;
