@@ -1,5 +1,9 @@
-# params-lib.pl
-# Functions for editing parameters common to many kinds of directive
+#!/usr/bin/perl
+# $Id: params-lib.pl,v 1.3 2005/04/16 14:30:21 jfranken Exp $
+# * Functions for editing parameters common to many kinds of directive
+#
+# File modified 2005-04-15 by Johannes Franken <jfranken@jfranken.de>:
+# * Added support for the client-update option of dhcpd 3.
 
 # display_params(&config, type)
 sub display_params
@@ -65,6 +69,31 @@ if ($config{'dhcpd_version'} >= 3) {
 		$uc ? "" : "checked", $text{'default'};
 	print "</td> </tr>\n";
 
+
+######## START CLIENT-UPDATES #####
+	# Inputs for allow/deny client-updates
+	if ($config{'dhcpd_version'} >= 3) {
+		local @adi = ( &find("allow", $_[0]), &find("deny", $_[0]),
+				   &find("ignore", $_[0]) );
+		local ($a, %vals);
+		foreach $a (@adi) {
+			$vals{$a->{'values'}->[0]} = $a;
+			}
+		local $cu = $vals{'client-updates'}->{'name'};
+		print "<tr> <td><b>$text{'plib_clientupdates'}</b></td> <td colspan=3>\n";
+		printf "<input type=radio name=clientupdates value=allow %s> %s\n",
+			$cu eq 'allow' ? "checked" : "", $text{'plib_allow'};
+		printf "<input type=radio name=clientupdates value=deny %s> %s\n",
+			$cu eq 'deny' ? "checked" : "", $text{'plib_deny'};
+		printf "<input type=radio name=clientupdates value=ignore %s> %s\n",
+			$cu eq 'ignore' ? "checked" : "", $text{'plib_ignore'};
+		printf "<input type=radio name=clientupdates value='' %s> %s\n",
+			$cu ? "" : "checked", $text{'default'};
+		print "</td> </tr>\n";
+
+	}
+######## END CLIENT-UPDATES #####
+
 	if ($_[1] eq 'subnet' || $_[1] eq 'shared-network' ||
 	    $_[1] eq 'global') {
 		# Inputs for authoratative
@@ -120,6 +149,11 @@ if ($config{'dhcpd_version'} >= 3) {
 		$in{'unclients'} ? [ { 'name' => $in{'unclients'},
 				       'values' => [ 'unknown-clients' ] } ]
 				 : [ ], $_[1], 1);
+	&save_directive($_[0],
+		$vals{'client-updates'} ? [ $vals{'client-updates'} ] : [ ],
+		$in{'clientupdates'} ? [ { 'name' => $in{'clientupdates'},
+				       'values' => [ 'client-updates' ] } ]
+				 : [ ], $_[1], 1);
 	if (defined($in{'auth'})) {
 		if ($in{'auth'}) {
 			&save_directive($_[0], "authoritative",
