@@ -29,7 +29,7 @@ else {
 
 	if ($dump->{'email'}) {
 		# Save output for mailing
-		$temp = &tempname();
+		$temp = &transname();
 		open(OUT, ">$temp");
 		}
 	else {
@@ -72,7 +72,7 @@ if ($out && $dump->{'email'} && &foreign_check("mailboxes")) {
 	# Construct the email
 	&foreign_require("mailboxes", "mailboxes-lib.pl");
 	$host = &get_system_hostname();
-	@dirs = &dump_directories($d);
+	@dirs = &dump_directories($dump);
 	$dirs = join(", ", @dirs);
 	local $subject = $dump->{'subject'} ||
 			 "Backup of $dirs on $host";
