@@ -52,6 +52,12 @@ if ($_[0]->{'fs'} eq 'tar') {
 		$r == 2 ? "checked" : "", $text{'dump_rsh2'};
 	printf "<input name=rsh size=30 value='%s'></td> </tr>\n",
 		$r == 2 ? $_[0]->{'rsh'} : "";
+
+	# Password option for SSH
+	print "<tr> <td><b>",&hlink($text{'dump_pass'},"pass"),
+	      "</b></td>\n";
+	print "<td colspan=3>",&ui_password("pass", $_[0]->{'pass'}, 20),
+	      "</td> </tr>\n";
 	}
 }
 
@@ -169,6 +175,7 @@ if ($_[0]->{'fs'} eq 'tar') {
 			&error($text{'dump_ersh'});
 		$_[0]->{'rsh'} = $in{'rsh'};
 		}
+	$_[0]->{'pass'} = $in{'pass'};
 	$in{'label'} =~ /^\S*$/ && length($in{'label'}) < 16 ||
 		&error($text{'dump_elabel'});
 	$_[0]->{'label'} = $in{'label'};
@@ -262,22 +269,12 @@ else {
 
 &system_logged("sync");
 sleep(1);
-&additional_log('exec', undef, $cmd);
-open(CMD, "$cmd 2>&1 </dev/null |");
-while(<CMD>) {
-	if ($_[2]) {
-		print $fh &html_escape($_);
-		}
-	else {
-		print $fh $_;
-		}
-	}
-close(CMD);
+local $got = &run_ssh_command($cmd, $fh, $_[2], $_[0]->{'pass'});
 if ($_[0]->{'multi'} && $_[0]->{'fs'} eq 'tar') {
 	# Run multi-file switch command one last time
 	system("$multi_cmd $_[0]->{'id'} >/dev/null 2>&1");
 	}
-return $? ? 0 : 1;
+return $got ? 0 : 1;
 }
 
 # dump_dest(&dump)
@@ -334,6 +331,12 @@ if ($_[0] eq 'tar') {
 	printf "<input name=rsh size=30 value='%s'></td> </tr>\n",
 		$r == 2 ? $_[1]->{'rsh'} : "";
 
+	# Password option for SSH
+	print "<tr> <td><b>",&hlink($text{'dump_pass'},"pass"),
+	      "</b></td>\n";
+	print "<td colspan=3>",&ui_password("pass", $_[0]->{'pass'}, 20),
+	      "</td> </tr>\n";
+
 	print "<tr> <td><b>",&hlink($text{'restore_files'},"rfiles"),
 	      "</b></td>\n";
 	print "<td colspan=3><input type=radio name=files_def value=1 checked> ",
@@ -484,7 +487,7 @@ sub restore_backup
 local ($fh, $fpid) = &foreign_call("proc", "pty_process_exec", "cd '$in{'dir'}' ; $_[1]");
 local $donevolume;
 while(1) {
-	local $rv = &wait_for($fh, "(next volume #)", "(set owner.mode for.*\\[yn\\])", "((.*)\\[yn\\])", "(.*\\n)");
+	local $rv = &wait_for($fh, "(next volume #)", "(set owner.mode for.*\\[yn\\])", "((.*)\\[yn\\])", "password:", "yes\\/no", "(.*\\n)");
 	last if ($rv < 0);
 	print &html_escape($matches[1]);
 	if ($rv == 0) {
@@ -502,6 +505,12 @@ while(1) {
 		return &text('restore_equestion',
 			     "<tt>$matches[2]</tt>");
 		}
+	elsif ($rv == 3) {
+		syswrite($fh, "$in{'pass'}\n");
+		}
+	elsif ($rv == 4) {
+		syswrite($fh, "yes\n");
+		}
 	}
 close($fh);
 waitpid($fpid, 0);
