@@ -9,6 +9,13 @@ push(@rv, "xfs") if (&has_command("xfsdump"));
 return @rv;
 }
 
+# multiple_directory_support(fs)
+# Returns 1 if some filesystem dump supports multiple directories
+sub multiple_directory_support
+{
+return 0;
+}
+
 # dump_form(&dump)
 sub dump_form
 {
