@@ -28,6 +28,7 @@ else { $got_net_imap++; }
 
 $secret_file = "/etc/ldap.secret";
 $samba_class = $config{'samba_class'} || "sambaAccount";
+$samba_group_class = $config{'samba_gclass'} || "sambaGroup";
 $cyrus_class = $config{'imap_class'} || "SuSEeMailObject";
 
 if ($config{'charset'}) {
@@ -58,20 +59,29 @@ return \%nss_config_cache;
 sub ldap_connect
 {
 local $conf = &get_nss_config();
-local ($host, $port);
+local (@hosts, $port);
 if ($conf) {
-	$host = $conf->{'host'};
+	@hosts = split(/[ ,]+/, $conf->{'host'});
 	$port = $conf->{'port'};
 	}
 else {
-	$host = $config{'ldap_host'};
+	@hosts = split(/[ ,]+/, $config{'ldap_host'});
 	$port = $config{'ldap_port'};
 	}
+@hosts = ( "localhost" ) if (!@hosts);
 $port ||= 389;
-local $ldap = Net::LDAP->new($host, port => $port);
-if (!$ldap) {
-	local $err = &text('conn_econn',
-			   "<tt>$host</tt>","<tt>$port</tt>");
+local ($ldap, $err);
+foreach $host (@hosts) {
+	$ldap = Net::LDAP->new($host, port => $port);
+	if (!$ldap) {
+		$err = &text('conn_econn',
+				   "<tt>$host</tt>","<tt>$port</tt>");
+		}
+	else {
+		last;
+		}
+	}
+if ($err) {
 	if ($_[0]) { return $err; }
 	else { &error($err); }
 	}
@@ -307,6 +317,7 @@ local $ldap = &ldap_connect();
 local $base = &get_group_base();
 local $_[0]->{'dn'} = "cn=$_[0]->{'group'},$base";
 local @classes = ( "posixGroup" );
+push(@classes, split(/\s+/, $config{'gother_class'}));
 local @attrs = &group_to_dn($_[0]);
 push(@attrs, @{$_[0]->{'ldap_attrs'}});
 push(@attrs, "objectClass" => \@classes);
@@ -382,6 +393,8 @@ if ($_[0]->get_value("uid")) {
 			'inactive' => $_[0]->get_value("shadowInactive") || "",
 		      );
 	$user{'pass'} =~ s/^{[a-z0-9]+}//i;
+	$user{'all_ldap_attrs'} = { map { lc($_), $_[0]->get_value($_) }
+					$_[0]->attributes() };
 	return %user;
 	}
 else {
@@ -389,8 +402,8 @@ else {
 	local %group = ( 'dn' => $_[0]->dn(),
 			 'group' => $_[0]->get_value("cn"),
 			 'gid' => $_[0]->get_value("gidNumber"),
-			 'pass' => $_[0]->get_value("userPassword"),
-			 'members' => join(",", @members),
+			 'pass' => $_[0]->get_value("userPassword") || "",
+			 'members' => join(",", @members) || "",
 			);
 	return %group;
 	}
@@ -499,47 +512,6 @@ foreach $k (keys %pmap) {
 return @rv;
 }
 
-# substitute_template(text, &hash)
-# Given some text and a hash reference, for each ocurrance of $FOO or ${FOO} in
-# the text replaces it with the value of the hash key foo
-sub substitute_template
-{
-# Add some extra fixed parameters to the hash
-local %hash = %{$_[1]};
-$hash{'hostname'} = &get_system_hostname();
-
-# Actually do the substition
-local $rv = $_[0];
-local $s;
-foreach $s (keys %hash) {
-	local $us = uc($s);
-	local $sv = $hash{$s};
-	$rv =~ s/\$\{\Q$us\E\}/$sv/g;
-	$rv =~ s/\$\Q$us\E/$sv/g;
-	if ($sv) {
-		$rv =~ s/\$\{IF-\Q$us\E\}(\n?)([\000-\377]*?)\$\{ELSE-\Q$us\E\}(\n?)([\000-\377]*?)\$\{ENDIF-\Q$us\E\}(\n?)/\2/g;
-		$rv =~ s/\$\{IF-\Q$us\E\}(\n?)([\000-\377]*?)\$\{ENDIF-\Q$us\E\}(\n?)/\2/g;
-
-		$rv =~ s/\$IF-\Q$us\E(\n?)([\000-\377]*?)\$ELSE-\Q$us\E(\n?)([\000-\377]*?)\$ENDIF-\Q$us\E(\n?)/\2/g;
-		$rv =~ s/\$IF-\Q$us\E(\n?)([\000-\377]*?)\$ENDIF-\Q$us\E(\n?)/\2/g;
-		}
-	else {
-		$rv =~ s/\$\{IF-\Q$us\E\}(\n?)([\000-\377]*?)\$\{ELSE-\Q$us\E\}(\n?)([\000-\377]*?)\$\{ENDIF-\Q$us\E\}(\n?)/\4/g;
-		$rv =~ s/\$\{IF-\Q$us\E\}(\n?)([\000-\377]*?)\$\{ENDIF-\Q$us\E\}(\n?)//g;
-
-		$rv =~ s/\$IF-\Q$us\E(\n?)([\000-\377]*?)\$ELSE-\Q$us\E(\n?)([\000-\377]*?)\$ENDIF-\Q$us\E(\n?)/\4/g;
-		$rv =~ s/\$IF-\Q$us\E(\n?)([\000-\377]*?)\$ENDIF-\Q$us\E(\n?)//g;
-		}
-	}
-
-# Remove un-replaced IF blocks
-$rv =~ s/\$\{IF-([A-Z0-9_]+)\}(\n?)([\000-\377]*?)\$\{ELSE-$1\}(\n?)([\000-\377]*?)\$\{ENDIF-$1\}(\n?)/\5/g;
-$rv =~ s/\$IF-([A-Z0-9_]+)(\n?)([\000-\377]*?)\$ELSE-$1(\n?)([\000-\377]*?)\$ENDIF-$1(\n?)/\5/g;
-$rv =~ s/\$\{IF-([A-Z0-9_]+)\}(\n?)([\000-\377]*?)\$\{ENDIF-$1\}(\n?)//g;
-$rv =~ s/\$IF-([A-Z0-9_]+)(\n?)([\000-\377]*?)\$ENDIF-$1(\n?)//g;
-return $rv;
-}
-
 # build_user_used([&uid-hash], [&shell-list], [&username-hash])
 # Fills in a hash with used UIDs and shells
 sub build_user_used
@@ -722,7 +694,172 @@ elsif ($mconfig{'home_style'} == 5) {
 	}
 }
 
+# imap_error(text, &rv)
+sub imap_error
+{
+&error_setup(undef);
+&error(&text('usave_eimap', $_[0], $_[1]->{'Text'}));
+}
 
+# setup_imap(&user)
+sub setup_imap
+{
+# Check if the user already exists
+local $imap = &imap_connect();
+local $rv = $imap->status("user.".$_[0]->{'user'}, "messages");
+if ($rv->{'Status'} eq 'ok') {
+	# Already exists, so do nothing
+	$imap->logout();
+	}
+else {
+	# Create the user on the IMAP server
+	$rv = $imap->create("user.".$_[0]->{'user'});
+	$rv->{'Status'} eq 'ok' ||
+		&imap_error($text{'usave_eicreate'}, $rv);
+
+	# Grant all rights to admin user
+	$rv = $imap->setacl("user.".$_[0]->{'user'},
+			    $config{'imap_login'}, "lrswipcda");
+	$rv->{'Status'} eq 'ok' ||
+		&imap_error($text{'usave_eiacl'}, $rv);
+
+	if ($config{'quota'}) {
+		# Set his IMAP quota
+		$rv = $imap->setquota("user.".$_[0]->{'user'},
+				      "STORAGE", $config{'quota'});
+		$rv->{'Status'} eq 'ok' ||
+			&imap_error($text{'usave_eiquota'}, $rv);
+		}
+
+	# Subscribe the user to his inbox by logging in
+	# as him
+	$imap->logout();
+	$uimap = new Net::IMAP($config{'imap_host'});
+	$rv = $uimap->login($_[0]->{'user'}, $_[0]->{'plainpass'});
+	$rv->{'Status'} eq 'ok' ||
+		&imap_error($text{'usave_eilogin'}, $rv);
+	$rv = $uimap->subscribe("INBOX");
+	$rv->{'Status'} eq 'ok' ||
+		&imap_error(&text('usave_eisub', "INBOX"), $rv);
+
+	foreach $f (split(/\t+/, $config{'imap_folders'})) {
+		local $fp = $config{'imap_folderalt'} ?
+				"user.$_[0]->{'user'}.$f" : $f;
+		$rv = $uimap->create($fp);
+		$rv->{'Status'} eq 'ok' ||
+		    &imap_error(&text('usave_eifolder',$f),$rv);
+		$rv = $uimap->subscribe($fp);
+		$rv->{'Status'} eq 'ok' ||
+		    &imap_error(&text('usave_eisub', $f), $rv);
+		}
+
+	$uimap->logout();
+	}
+
+# Re-connect for later LDAP operations
+$ldap = &ldap_connect();
+}
+
+# setup_addressbook(&user)
+sub setup_addressbook
+{
+local $ldap = &ldap_connect();
+local $rv = $ldap->add("ou=$_[0]->{'user'}, $config{'addressbook'}", attr =>
+		 [ "ou" => $_[0]->{'user'},
+		   "objectClass" => [ "top", "organizationalUnit" ]
+		 ] );
+if ($rv->code) {
+	&error(&text('usave_ebook', $rv->error));
+	}
+$ldap->unbind();
+}
+
+# other_modules(function, arg, ...)
+# Call some function in the useradmin_update.pl file in other modules
+sub other_modules
+{
+return &useradmin::other_modules(@_);
+}
+
+# lock_user_files()
+# Does nothing, because no locking is needed for LDAP
+sub lock_user_files
+{
+}
+
+# unlock_user_files()
+# Does nothing, because no locking is needed for LDAP
+sub unlock_user_files
+{
+}
+
+# in_schema(schema, attrname)
+sub in_schema
+{
+return $_[0] && $_[0]->attribute($_[1]);
+}
+
+# extra_fields_input(fields-list, &user|&group)
+sub extra_fields_input
+{
+local ($fields, $uinfo) = @_;
+local @fields = map { [ split(/\s+/, $_, 2) ] } split(/\t/, $fields);
+if (@fields) {
+	print "<table border width=100%>\n";
+	print "<tr $tb> <td><b>$text{'uedit_fields'}</b></td> </tr>\n";
+	print "<tr $cb> <td><table width=100%>\n";
+	local $i = 0;
+	local $f;
+	foreach $f (@fields) {
+		local ($multi) = ($f->[0] =~ s/\+$//);
+		print "<tr>\n" if ($i%2 == 0);
+
+		print "<td valign=top width=25%><b>$f->[1]</b></td>\n";
+		print "<td valign=top width=25%>\n";
+		local @v;
+		if ($in{'new'}) {
+			$v[0] = $in{lc($f->[0])};
+			}
+		else {
+			@v = $uinfo->get_value($f->[0]);
+			}
+		if ($config{'multi_fields'} || @v > 1 || $multi) {
+			print "<textarea name=field_$i rows=3 cols=25>\n";
+			print join("\n", @v);
+			print "</textarea>\n";
+			}
+		else {
+			printf "<input name=field_%d size=25 value='%s'>\n",
+				$i, $v[0];
+			}
+		print "</td>\n";
+
+		print "</tr>\n" if ($i++%2 == 1);
+		}
+	print "</table></td></tr></table><p>\n";
+	}
+}
+
+# parse_extra_fields(fields-list, &props, &rprops)
+sub parse_extra_fields
+{
+local ($fields, $props, $rprops) = @_;
+local @fields = map { [ split(/\s+/, $_, 2) ] } split(/\t/, $fields);
+local $i = 0;
+local $f;
+foreach $f (@fields) {
+	$f->[0] =~ s/\+$//;
+	if ($in{"field_$i"} eq "") {
+		push(@$rprops, $f->[0]);
+		}
+	else {
+		$in{"field_$i"} =~ s/\r//g;
+		local @v = split(/\n/, $in{"field_$i"});
+		push(@$props, $f->[0], @v == 1 ? $v[0] : \@v);
+		}
+	$i++;
+	}
+}
 
 1;
 
