@@ -105,6 +105,7 @@ else {
 	else {
 		&system_logged("cp $_[1]->{'ppd'} $drv");
 		}
+	chmod(0777, $drv);
 	$cups_driver_options = $_[1]->{'opts'};	# for modify_printer
 	return $drv;
 	}
@@ -153,14 +154,16 @@ if ($outofdate || scalar(keys %cache) != scalar(@files)) {
 	local %donecache;
 	foreach $f (@files) {
 		local $ppd = &parse_cups_ppd($f);
-		$cache{$f} = $ppd->{'NickName'}
-			if (!$donecache{$ppd->{'NickName'}}++);
+		local $nn = $ppd->{'NickName'};
+		$cache{$f} = $donecache{$nn} ? "duplicate" : $ppd->{'NickName'};
+		$donecache{$nn}++;
 		}
 	&write_file("$module_config_directory/ppd-cache", \%cache);
 	}
 local %done;
 foreach $f (sort { $cache{$a} cmp $cache{$b} } keys %cache) {
-	if ($cache{$f} && !$done{$cache{$f}}++) {
+	if ($cache{$f} && $cache{$f} ne "duplicate" &&
+	    !$done{$cache{$f}}++) {
 		printf "<option value=%s %s>%s\n",
 			$f, $_[1]->{'nick'} eq $cache{$f} ? 'selected' : '',
 			$cache{$f};
