@@ -5,6 +5,7 @@ require 'mailboxes-lib.pl';
 # Output HTML for editing security options for the sendmail module
 sub acl_security_form
 {
+# Users whose mail can be read
 print "<tr> <td valign=top><b>$text{'acl_read'}</b></td> <td colspan=3>\n";
 printf "<input type=radio name=mmode value=0 %s> $text{'acl_none'}\n",
 	$_[0]->{'mmode'} == 0 ? "checked" : "";
@@ -29,9 +30,10 @@ printf "<input name=muserse size=40 value='%s'> %s<br>\n",
 
 printf "<input type=radio name=mmode value=5 %s> $text{'acl_usersg'}\n",
 	$_[0]->{'mmode'} == 5 ? "checked" : "";
-printf "<input name=musersg size=8 value='%s'> %s\n",
-	$_[0]->{'mmode'} == 5 ? scalar(getgrgid($_[0]->{'musers'})) : "",
-	&group_chooser_button("musersg");
+printf "<input name=musersg size=30 value='%s'> %s\n",
+	$_[0]->{'mmode'} == 5 ? join(" ", map { scalar(getgrgid($_)) }
+				     split(/\s+/, $_[0]->{'musers'})) : "",
+	&group_chooser_button("musersg", 1);
 printf "<input type=checkbox name=msec value=1 %s> %s<br>\n",
 	$_[0]->{'msec'} ? "checked" : "", $text{'acl_sec'};
 
@@ -47,6 +49,12 @@ printf "<input type=radio name=mmode value=6 %s> $text{'acl_usersm'}\n",
 printf "<input name=musersm size=15 value='%s'></td> </tr>\n",
 	$_[0]->{'mmode'} == 6 ? $_[0]->{'musers'} : "";
 
+# Directory for arbitrary files
+print "<tr> <td valign=top><b>$text{'acl_dir'}</b></td> <td colspan=3>\n";
+print &ui_opt_textbox("dir", $_[0]->{'dir'}, 40, $text{'acl_dirauto'}."<br>");
+print "</td> </tr>\n";
+
+# Allowed From: addresses
 print "<tr> <td valign=top><b>$text{'acl_from'}</b></td> <td colspan=3>\n";
 printf "<input type=radio name=fmode value=0 %s> $text{'acl_any'}<br>\n",
 	$_[0]->{'fmode'} == 0 ? "checked" : "";
@@ -97,7 +105,8 @@ sub acl_security_save
 $_[0]->{'mmode'} = $in{'mmode'};
 $_[0]->{'musers'} = $in{'mmode'} == 2 ? $in{'musers'} :
 		    $in{'mmode'} == 3 ? $in{'muserse'} :
-		    $in{'mmode'} == 5 ? scalar(getgrnam($in{'musersg'})) :
+		    $in{'mmode'} == 5 ? join(" ", map { scalar(getgrnam($_)) }
+					     split(/\s+/, $in{'musersg'})) :
 		    $in{'mmode'} == 6 ? $in{'musersm'} :
 		    $in{'mmode'} == 7 ? $in{'musersu1'} : "";
 $_[0]->{'musers2'} = $in{'mmode'} == 7 ? $in{'musersu2'} : "";
@@ -110,5 +119,6 @@ $_[0]->{'fromname'} = $in{'fromname'};
 $_[0]->{'attach'} = $in{'attach_def'} ? -1 : $in{'attach'};
 $_[0]->{'canattach'} = $in{'canattach'};
 $_[0]->{'candetach'} = $in{'candetach'};
+$_[0]->{'dir'} = $in{'dir_def'} ? undef : $in{'dir'};
 }
 
