@@ -36,18 +36,7 @@ elsif ($_[2]->{'type'} == 2) {
 	local %onserver = map { &safe_uidl($_), 1 } @uidl;
 
 	# Work out what range we want
-	local ($start, $end);
-	if (!defined($_[0])) {
-		$start = 0; $end = @uidl-1;
-		}
-	elsif ($_[1] < 0) {
-		$start = @uidl+$_[1]-1; $end = @uidl+$_[0]-1;
-		$start = $start<0 ? 0 : $start;
-		}
-	else {
-		$start = $_[0]; $end = $_[1];
-		$end = @uidl-1 if ($end >= @uidl);
-		}
+	local ($start, $end) = &compute_start_end($_[0], $_[1], scalar(@uidl));
 	local @rv = map { undef } @uidl;
 
 	# For each message in the range, get the headers or body
@@ -164,18 +153,7 @@ elsif ($_[2]->{'type'} == 4) {
 	return () if (!$count);
 
 	# Work out what range we want
-	local ($start, $end);
-	if (!defined($_[0])) {
-		$start = 0; $end = $count-1;
-		}
-	elsif ($_[1] < 0) {
-		$start = $count+$_[1]-1; $end = $count+$_[0]-1;
-		$start = $start<0 ? 0 : $start;
-		}
-	else {
-		$start = $_[0]; $end = $_[1];
-		$end = $count-1 if ($end >= $count);
-		}
+	local ($start, $end) = &compute_start_end($_[0], $_[1], $count);
 	local @mail = map { undef } (0 .. $count-1);
 
 	# Get the headers or body of messages in the specified range
@@ -217,18 +195,7 @@ elsif ($_[2]->{'type'} == 5) {
 		}
 
 	# Work out what range we need
-	local ($start, $end);
-	if (!defined($_[0])) {
-		$start = 0; $end = $count-1;
-		}
-	elsif ($_[1] < 0) {
-		$start = $count+$_[1]-1; $end = $count+$_[0]-1;
-		$start = $start<0 ? 0 : $start;
-		}
-	else {
-		$start = $_[0]; $end = $_[1];
-		$end = $count-1 if ($end >= $count);
-		}
+	local ($start, $end) = &compute_start_end($_[0], $_[1], $count);
 
 	# Fetch the needed part of each sub-folder
 	local $pos = 0;
@@ -256,6 +223,73 @@ elsif ($_[2]->{'type'} == 5) {
 
 	return @mail;
 	}
+elsif ($_[2]->{'type'} == 6) {
+	# A virtual folder, which just contains indexes into other folders
+	local $mems = $folder->{'members'};
+	local ($start, $end) = &compute_start_end($_[0], $_[1], scalar(@$mems));
+
+	# Work out which folders we need, and how much
+	local (%frange, %namemap);
+	local $i;
+	for($i=$start; $i<=$end; $i++) {
+		local $sf = $mems->[$i]->[0];
+		local $sfn = &folder_name($sf);
+		local $idx = $mems->[$i]->[1];
+		if ($frange{$sfn}) {
+			$frange{$sfn}->[0] = $idx if ($idx < $frange{$sfn}->[0]);
+			$frange{$sfn}->[1] = $idx if ($idx > $frange{$sfn}->[1]);
+			}
+		else {
+			$frange{$sfn} = [ $idx, $idx ];
+			}
+		$namemap{$sfn} = $sf;
+		}
+
+	# Get all the needed folders
+	local %sfs;
+	local $sfn;
+	foreach $sfn (keys %frange) {
+		local $sf = $namemap{$sfn};
+		local @submail = &mailbox_list_mails($frange{$sfn}->[0],
+						     $frange{$sfn}->[1],
+						     $sf, $_[3]);
+		$sfs{$sfn} = \@submail;
+		}
+
+	# Construct the results
+	local @mail = map { undef } (0 .. @$mems-1);
+	for($i=$start; $i<=$end; $i++) {
+		local $sf = $mems->[$i]->[0];
+		local $sfn = &folder_name($sf);
+		local $idx = $mems->[$i]->[1];
+		$mail[$i] = $sfs{$sfn}->[$idx];
+		$mail[$i]->{'subidx'} = $mail[$i]->{'idx'};
+		$mail[$i]->{'idx'} = $i;
+		$mail[$i]->{'subfolder'} = $sf;
+		}
+
+	return @mail;
+	}
+}
+
+# compute_start_end(start, end, count)
+sub compute_start_end
+{
+local ($start, $end, $count) = @_;
+if (!defined($start)) {
+	return (0, $count-1);
+	}
+elsif ($end < 0) {
+	local $rstart = $count+$_[1]-1;
+	local $rend = $count+$_[0]-1;
+	$rstart = $rstart < 0 ? 0 : $rstart;
+	return ($rstart, $rend);
+	}
+else {
+	local $rend = $_[1];
+	$rend = $count - 1 if ($rend >= $count);
+	return ($start, $rend);
+	}
 }
 
 # mailbox_search_mail(&fields, andmode, &folder, [&limit])
@@ -374,6 +408,20 @@ elsif ($_[2]->{'type'} == 5) {
 		}
 	return reverse(@mail);
 	}
+elsif ($_[2]->{'type'} == 6) {
+	# Just run a search on the sub-mails
+	local @rv;
+	local ($min, $max);
+	if ($_[3] && $_[3]->{'latest'}) {
+		$min = -1;
+		$max = -$_[3]->{'latest'};
+		}
+	local $mail;
+	foreach $mail (&mailbox_list_mails($min, $max, $_[2])) {
+		push(@rv, $mail) if ($mail && &mail_matches($_[0],$_[1],$mail));
+		}
+	return @rv;
+	}
 }
 
 # mailbox_delete_mail(&folder, mail, ...)
@@ -445,6 +493,14 @@ elsif ($f->{'type'} == 5) {
 		$sm->{'idx'} = $oldidx;
 		}
 	}
+elsif ($f->{'type'} == 6) {
+	# Just take out of the virtual index
+	local $sm;
+        foreach $sm (sort { $b->{'idx'} <=> $a->{'idx'} } @_) {
+		splice(@{$f->{'members'}}, $sm->{'idx'}, 1);
+		}
+	&save_folder($f, $f);
+	}
 }
 
 # mailbox_empty_folder(&folder)
@@ -502,6 +558,11 @@ elsif ($f->{'type'} == 5) {
 		&mailbox_empty_folder($sf);
 		}
 	}
+elsif ($f->{'type'} == 6) {
+	# Just clear the virtual index
+	$f->{'members'} = [ ];
+	&save_folder($f);
+	}
 }
 
 # mailbox_move_mail(&source, &dest, mail, ...)
@@ -548,11 +609,20 @@ sub mailbox_copy_mail
 local $src = shift(@_);
 local $dst = shift(@_);
 local $now = time();
-local $hn = &get_system_hostname();
 &create_folder_maildir($dst);
 local $m;
-foreach $m (@_) {
-	&write_mail_folder($m, $dst);
+if ($dst->{'type'} == 6) {
+	# Add this mail to the index of the virtual folder
+	foreach $m (@_) {
+		push(@{$dst->{'members'}}, [ $src, $m->{'idx'} ]);
+		}
+	&save_folder($dst);
+	}
+else {
+	# Just write to destination folder
+	foreach $m (@_) {
+		&write_mail_folder($m, $dst);
+		}
 	}
 }
 
@@ -604,7 +674,7 @@ elsif ($_[1]->{'type'} == 4) {
 	local $h = $rv[1];
 
 	# Create a temp file and use it to create the IMAP command
-	local $temp = &tempname();
+	local $temp = &transname();
 	&send_mail($_[0], $temp, $_[2], 1);
 	open(TEMP, $temp);
 	local $text;
@@ -620,6 +690,10 @@ elsif ($_[1]->{'type'} == 5) {
 	local @sf = @{$_[1]->{'subfolders'}};
 	&write_mail_folder($_[0], $sf[$#sf], $_[2]);
 	}
+elsif ($_[1]->{'type'} == 6) {
+	# Add mail to first sub-folder, and to virtual index
+	&error("Cannot add mail to virtual folders");
+	}
 }
 
 # mailbox_modify_mail(&oldmail, &newmail, &folder, textonly)
@@ -638,7 +712,7 @@ elsif ($_[2]->{'type'} == 0) {
 	# Modify the mail file
 	&modify_mail($_[2]->{'file'}, $_[0], $_[1], $_[3]);
 	}
-elsif ($_[2]->{'type'} == 5) {
+elsif ($_[2]->{'type'} == 5 || $_[2]->{'type'} == 6) {
 	# Modify in the appropriate folder
 	local $oldoldidx = $_[0]->{'idx'};
 	$_[0]->{'idx'} = $_[0]->{'subidx'};
@@ -695,6 +769,10 @@ elsif ($_[0]->{'type'} == 4) {
 		}
 	return $rv[2];
 	}
+elsif ($_[0]->{'type'} == 6) {
+	# A virtual folder, for which the size is known
+	return scalar(@{$_[0]->{'members'}});
+	}
 }
 
 # pop3_login(&folder)
@@ -893,7 +971,7 @@ return @rv;
 # lock_folder(&folder)
 sub lock_folder
 {
-return if ($_[0]->{'remote'});
+return if ($_[0]->{'remote'} || $_[0]->{'type'} == 5 || $_[0]->{'type'} == 6);
 local $f = $_[0]->{'file'} ? $_[0]->{'file'} :
 	   $_[0]->{'type'} == 0 ? &user_mail_file($remote_user) :
 				  $qmail_maildir;
@@ -1105,7 +1183,7 @@ sub html_to_text
 local ($h2, $lynx);
 if (($h2 = &has_command("html2text")) || ($lynx = &has_command("lynx"))) {
 	# Can use a commonly available external program
-	local $temp = &tempname().".html";
+	local $temp = &transname().".html";
 	open(TEMP, ">$temp");
 	print TEMP $_[0];
 	close(TEMP);
@@ -1156,7 +1234,7 @@ return $sel;
 }
 
 # folder_size(&folder, ...)
-# Sets the 'size' field of one or more folders
+# Sets the 'size' field of one or more folders, and returns the total
 sub folder_size
 {
 local ($f, $total);
@@ -1174,6 +1252,7 @@ foreach $f (@_) {
 			local $mf;
 			opendir(QDIR, "$f->{'file'}/$qd");
 			while($mf = readdir(QDIR)) {
+				next if ($mf eq "." || $mf eq "..");
 				local @st = stat("$f->{'file'}/$qd/$mf");
 				$f->{'size'} += $st[7];
 				}
@@ -1186,6 +1265,7 @@ foreach $f (@_) {
 		$f->{'size'} = 0;
 		opendir(MHDIR, $f->{'file'});
 		while($mf = readdir(MHDIR)) {
+			next if ($mf eq "." || $mf eq "..");
 			local @st = stat("$f->{'file'}/$mf");
 			$f->{'size'} += $st[7];
 			}
@@ -1252,12 +1332,15 @@ closedir(DIR);
 foreach $f (@files) {
 	next if ($f eq "." || $f eq ".." || $f =~ /\.lock$/i ||
 		 $f eq "cur" || $f eq "tmp" || $f eq "new" ||
-		 $f =~ /^\.imap/i);
+		 $f =~ /^\.imap/i || $f eq ".customflags" ||
+		 $f eq "dovecot-uidlist");
 	local $p = "$_[0]/$f";
 	if ($_[1] || !-d $p || -d "$p/cur") {
 		push(@rv, $p);
 		}
-	else {
+	# Add all sub-files, to handle Maildir mailboxes that have their own
+	# sub-folders
+	if (-d "$p/cur") {
 		push(@rv, &recursive_files($p));
 		}
 	}
@@ -1290,45 +1373,31 @@ return "cid:$_[0]" if (!$cont);
 return "$_[2]&attach=$cont->{'idx'}";
 }
 
-$search_directory = "$user_module_config_directory/searches";
-
-# save_mailbox_search(&mails)
-# Saves the list of messages that matches some search, and returns a new
-# search ID. Also checks for expired searches and deletes them.
-sub save_mailbox_search
-{
-local $sid = time().".".$$;
-open(SEARCH, ">$search_directory/$sid");
-foreach $m (@{$_[0]}) {
-	print SEARCH "$m->{'index'}\n";
-	}
-close(SEARCH);
-
-# XXX remove old
-
-return $sid;
-}
-
-# get_mailbox_search(id)
-# Returns a list of mail indexes that matched some previous search
-sub get_mailbox_search
-{
-}
-
-# quoted_message(&mail, quote-mode, sig)
+# quoted_message(&mail, quote-mode, sig, 0=any,1=text,2=html)
 # Returns the quoted text, html-flag and body attachment
 sub quoted_message
 {
-local ($mail, $qu, $sig) = @_;
-local ($plainbody, $htmlbody) = find_body($mail, $config{'view_html'});
+local ($mail, $qu, $sig, $bodymode) = @_;
+local $mode = $bodymode == 1 ? 1 :
+	      $bodymode == 2 ? 2 :
+	      defined(%userconfig) ? $userconfig{'view_html'} :
+				     $config{'view_html'};
+local ($plainbody, $htmlbody) = &find_body($mail, $mode);
 local ($quote, $html_edit, $body);
 local $cfg = defined(%userconfig) ? \%userconfig : \%config;
 local @writers = &split_addresses($mail->{'header'}->{'from'});
 local $writer = ($writers[0]->[1] || $writers[0]->[0])." wrote ..";
+local $tm;
+if ($cfg->{'reply_date'} &&
+    ($tm = &parse_mail_date($_[0]->{'header'}->{'date'}))) {
+	local $tmstr = &make_date($tm);
+	$writer = "On $tmstr $writer";
+	}
 local $qm = defined(%userconfig) ? $userconfig{'html_quote'}
 				 : $config{'html_quote'};
-if ($cfg->{'html_edit'} == 2 ||
-    $cfg->{'html_edit'} == 1 && $htmlbody) {
+if (($cfg->{'html_edit'} == 2 ||
+     $cfg->{'html_edit'} == 1 && $htmlbody) &&
+     $bodymode != 1) {
 	# Create quoted body HTML
 	if ($htmlbody) {
 		$body = $htmlbody;
@@ -1475,5 +1544,35 @@ eval { local $main::errors_must_die = 1; &send_mail($dmail); };
 return $to;
 }
 
+# find_named_folder(name, &folders, [&cache])
+# Finds a folder by ID, filename, server name or displayed name
+sub find_named_folder
+{
+local $rv;
+if ($_[2] && exists($_[2]->{$_[0]})) {
+	# In cache
+	$rv = $_[2]->{$_[0]};
+	}
+else {
+	# Need to lookup
+	($rv) = grep { $_->{'id'} eq $_[0] } @{$_[1]} if (!$rv);
+	($rv) = grep { $_->{'file'} eq $_[0] ||
+			     $_->{'server'} eq $_[0] } @{$_[1]} if (!$rv);
+	($rv) = grep { $_->{'name'} eq $_[0] } @{$_[1]} if (!$rv);
+	$_[2]->{$_[0]} = $rv if ($_[2]);
+	}
+return $rv;
+}
+
+# folder_name(&folder)
+# Returns a unique identifier for a folder, based on it's filename or ID
+sub folder_name
+{
+return $_[0]->{'id'} ||
+       $_[0]->{'file'} ||
+       $_[0]->{'server'} ||
+       $_[0]->{'name'};
+}
+
 1;
 
