@@ -1,7 +1,5 @@
 # postgresql-lib.pl
 # Common PostgreSQL functions
-# XXX dropping fields from schema table
-#	XXX maybe can only rename within schema?
 # XXX updating date field
 # XXX access control and schema tables
 
@@ -57,7 +55,7 @@ EOF
 # is a library problem
 sub is_postgresql_running
 {
-local $temp = &tempname();
+local $temp = &transname();
 local $host = $config{'host'} ? "-h $config{'host'}" : "";
 $host .= " -p $config{'port'}" if ($config{'port'});
 local $cmd;
@@ -98,14 +96,22 @@ else {
 # get_postgresql_version()
 sub get_postgresql_version
 {
-local $v = &execute_sql($config{'basedb'}, 'select version()');
-$v = $v->{'data'}->[0]->[0];
-if ($v =~ /postgresql\s+([0-9\.]+)/i) {
-	return $1;
-	}
-else {
-	return undef;
+local $main::error_must_die = 1;
+local $v = eval {
+	local $v = &execute_sql($config{'basedb'}, 'select version()');
+	$v = $v->{'data'}->[0]->[0];
+	if ($v =~ /postgresql\s+([0-9\.]+)/i) {
+		return $1;
+		}
+	else {
+		return undef;
+		}
+	};
+if (!$v) {
+	local $out = `$config{'psql'} -V 2>&1 </dev/null`;
+	return $out =~ /\s([0-9\.]+)$/ ? $1 : undef;
 	}
+return $v;
 }
 
 sub can_drop_fields
@@ -122,13 +128,19 @@ local $t = &execute_sql($config{'basedb'}, 'select * from pg_database order by d
 return map { $_->[0] } @{$t->{'data'}};
 }
 
+# supports_schemas(database)
+# Returns 1 if schemas are supported
+sub supports_schemas
+{
+local $t = &execute_sql($_[0], "select a.attname FROM pg_class c, pg_attribute a, pg_type t WHERE c.relname = 'pg_tables' and a.attnum > 0 and a.attrelid = c.oid and a.atttypid = t.oid and a.attname = 'schemaname' order by attnum");
+return $t->{'data'}->[0]->[0] ? 1 : 0;
+}
+
 # list_tables(database)
 # Returns a list of tables in some database
 sub list_tables
 {
-local @str = &table_structure($_[0], "pg_tables");
-local %fields = map { $_->{'field'}, 1 } @str;
-if ($fields{'schemaname'}) {
+if (&supports_schemas($_[0])) {
 	local $t = &execute_sql($_[0], 'select schemaname,tablename from pg_tables where tablename not like \'pg_%\' and tablename not like \'sql_%\' order by tablename');
 	return map { ($_->[0] eq "public" ? "" : $_->[0].".").$_->[1] } @{$t->{'data'}};
 	}
@@ -152,24 +164,54 @@ return sort { $a cmp $b } &unique(@types);
 # Returns a list of hashes detailing the structure of a table
 sub table_structure
 {
-local $tn = $_[1];
-$tn =~ s/^([^\.]+)\.//;
-local $t = &execute_sql($_[0], "select a.attnum, a.attname, t.typname, a.attlen, a.atttypmod, a.attnotnull, a.atthasdef FROM pg_class c, pg_attribute a, pg_type t WHERE c.relname = '$tn'    and a.attnum > 0     and a.attrelid = c.oid     and a.atttypid = t.oid order by attnum");
-local (@rv, $r);
-foreach $r (@{$t->{'data'}}) {
-	local $arr;
-	$arr++ if ($r->[2] =~ s/^_//);
-	local $sz = $r->[4] - 4;
-	if ($sz >= 65536 && $r->[2] =~ /numeric/i) {
-		$sz = int($sz/65536).",".($sz%65536);
+if (&supports_schemas($_[0])) {
+	# Find the schema and table
+	local ($tn, $ns);
+	if ($_[1] =~ /^(\S+)\.(\S+)$/) {
+		$ns = $1;
+		$tn = $2;
 		}
-	push(@rv, { 'field' => $r->[1],
-		    'arr' => $arr ? 'YES' : 'NO',
-		    'type' => $r->[4] < 0 ? $r->[2]
-					  : $r->[2]."($sz)",
-		    'null' => $r->[5] =~ /f|0/ ? 'YES' : 'NO' } );
+	else {
+		$ns = "public";
+		$tn = $_[1];
+		}
+	$tn =~ s/^([^\.]+)\.//;
+	local $t = &execute_sql($_[0], "select a.attnum, a.attname, t.typname, a.attlen, a.atttypmod, a.attnotnull, a.atthasdef FROM pg_class c, pg_attribute a, pg_type t, pg_namespace ns WHERE c.relname = '$tn' and ns.nspname = '$ns' and a.attnum > 0 and a.attrelid = c.oid and a.atttypid = t.oid and a.attname not like '%pg.dropped%' and c.relnamespace = ns.oid order by attnum");
+	local (@rv, $r);
+	foreach $r (@{$t->{'data'}}) {
+		local $arr;
+		$arr++ if ($r->[2] =~ s/^_//);
+		local $sz = $r->[4] - 4;
+		if ($sz >= 65536 && $r->[2] =~ /numeric/i) {
+			$sz = int($sz/65536).",".($sz%65536);
+			}
+		push(@rv, { 'field' => $r->[1],
+			    'arr' => $arr ? 'YES' : 'NO',
+			    'type' => $r->[4] < 0 ? $r->[2]
+						  : $r->[2]."($sz)",
+			    'null' => $r->[5] =~ /f|0/ ? 'YES' : 'NO' } );
+		}
+	return @rv;
+	}
+else {
+	# Just look by table name
+	local $t = &execute_sql($_[0], "select a.attnum, a.attname, t.typname, a.attlen, a.atttypmod, a.attnotnull, a.atthasdef FROM pg_class c, pg_attribute a, pg_type t WHERE c.relname = '$_[1]'    and a.attnum > 0     and a.attrelid = c.oid     and a.atttypid = t.oid order by attnum");
+	local (@rv, $r);
+	foreach $r (@{$t->{'data'}}) {
+		local $arr;
+		$arr++ if ($r->[2] =~ s/^_//);
+		local $sz = $r->[4] - 4;
+		if ($sz >= 65536 && $r->[2] =~ /numeric/i) {
+			$sz = int($sz/65536).",".($sz%65536);
+			}
+		push(@rv, { 'field' => $r->[1],
+			    'arr' => $arr ? 'YES' : 'NO',
+			    'type' => $r->[4] < 0 ? $r->[2]
+						  : $r->[2]."($sz)",
+			    'null' => $r->[5] =~ /f|0/ ? 'YES' : 'NO' } );
+		}
+	return @rv;
 	}
-return @rv;
 }
 
 # execute_sql(database, sql, [param, ...])
@@ -289,7 +331,7 @@ else {
 		}
 
 	# Call the psql program
-	local $temp = &tempname();
+	local $temp = &transname();
 	open(TEMP, ">$temp");
 	print TEMP "$postgres_login\n$postgres_pass\n";
 	close(TEMP);
@@ -373,7 +415,7 @@ else {
 		# Running a special \ command
 		local ( @titles, @row, @data, $rc, $emsgf, $emsg ) ;
 
-		$emsgf = &tempname();
+		$emsgf = &transname();
 		$rc = &system_logged ( "$cmd < $temp >$emsgf 2>&1");
 		$emsg  = `cat $emsgf` ;
 		unlink ( $emsgf ) ;
@@ -460,7 +502,20 @@ while(<HBA>) {
 	s/^\s*#.*$//g;
 	if ($_[0] >= 7.3) {
 		# New file format
-		if (/^\s*(host|hostssl)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)(\s+(\S+))?/) {
+		if (/^\s*(host|hostssl)\s+(\S+)\s+(\S+)\s+(\S+)\/(\S+)\s+(\S+)(\s+(\S+))?/) {
+			# Host/cidr format
+			push(@rv, { 'type' => $1,
+				    'index' => scalar(@rv),
+				    'line' => $lnum,
+				    'db' => $2,
+				    'user' => $3,
+				    'address' => $4,
+				    'cidr' => $5,
+				    'auth' => $6,
+				    'arg' => $8 } );
+			}
+		elsif (/^\s*(host|hostssl)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)(\s+(\S+))?/) {
+			# Host netmask format
 			push(@rv, { 'type' => $1,
 				    'index' => scalar(@rv),
 				    'line' => $lnum,
@@ -549,8 +604,10 @@ sub hba_line
 if ($_[0]->{'type'} eq 'host' || $_[0]->{'type'} eq 'hostssl') {
 	return join(" ", $_[0]->{'type'}, $_[0]->{'db'},
 			 ( $_[1] >= 7.3 ? ( $_[0]->{'user'} ) : ( ) ),
-			 $_[0]->{'address'},
-		         $_[0]->{'netmask'},
+			 ($_[0]->{'cidr'} eq '' ? 
+				 ( $_[0]->{'address'},
+				   $_[0]->{'netmask'} ) :
+				 ( "$_[0]->{'address'}/$_[0]->{'cidr'}" )),
 			 $_[0]->{'auth'},
 			 $_[0]->{'arg'} ? ( $_[0]->{'arg'} ) : () );
 	}
@@ -653,5 +710,31 @@ sub quotestr
 return "\"$_[0]\"";
 }
 
+# execute_sql_file(database, file, [user, pass])
+# Executes some file of SQL statements, and returns the exit status and output
+sub execute_sql_file
+{
+local ($db, $file, $user, $pass) = @_;
+local $cmd = "$config{'psql'} -u -f $file".
+	     ($config{'host'} ? " -h $config{'host'}" : "").
+	     ($config{'port'} ? " -h $config{'port'}" : "").
+	     " $db";
+local $ltemp = &transname();
+open(TEMP, ">$ltemp");
+if ($user) {
+	print TEMP "$user\n$pass\n";
+	}
+else {
+	print TEMP "$postgres_login\n$postgres_pass\n";
+	}
+close(TEMP);
+if ($postgres_sameunix && defined(getpwnam($postgres_login)) && !$user) {
+	$cmd = "su $postgres_login -c ".quotemeta($cmd);
+	}
+local $out = &backquote_logged("$cmd <$ltemp 2>&1");
+unlink($ltemp);
+return ($out =~ /ERROR/i ? 1 : 0, $out);
+}
+
 1;
 
