@@ -6,6 +6,7 @@ do '../web-lib.pl';
 &init_config();
 require '../ui-lib.pl';
 do 'secrets-lib.pl';
+%access = &get_module_acl();
 
 $options_pptp = $config{'pptp_ppp_options'} || "/etc/ppp/options.pptp";
 
@@ -202,7 +203,7 @@ sub get_pptpd_version
 {
 local $out = `$config{'pptpd'} -v 2>&1`;
 ${$_[0]} = $out;
-return $out =~ /PoPToP\s+v?(\S+)/i ? $1 : undef;
+return $out =~ /(PoPToP|pptpd)\s+v?(\S+)/i ? $2 : undef;
 }
 
 # apply_configuration()
