@@ -40,6 +40,7 @@ if ($has_xinetd) {
 		    $x->{'value'} eq $cvs_inet_name) {
 			# Found the entry
 			return { 'type' => 'xinetd',
+				 'user' => $q->{'user'}->[0],
 				 'command' => $q->{'server'}->[0],
 				 'args' => $q->{'server'}->[0]." ".
 					   $q->{'server_args'}->[0],
@@ -65,6 +66,7 @@ if ($has_inetd) {
 		    $i->[3] eq $cvs_inet_name) {
 			# Found the entry
 			return { 'type' => 'inetd',
+				 'user' => $i->[7],
 				 'command' => $i->[8],
 				 'args' => $i->[9],
 				 'active' => $i->[1],
