@@ -162,7 +162,7 @@ $_[0]->{'email'} = $in{'email'};
 $_[0]->{'ugrace'} = $in{'ugrace'};
 $_[0]->{'ggrace'} = $in{'ggrace'};
 $_[0]->{'diskspace'} = $in{'diskspace'};
-$_[0]->{'maxblocks'} = $in{'maxblocks'};
+$_[0]->{'maxblocks'} = $in{'maxblocks_def'} ? undef : $in{'maxblocks'};
 $_[0]->{'maxfiles'} = $in{'maxfiles'};
 }
 
