@@ -14,7 +14,7 @@ open(FILE, $f);
 while(<FILE>) { $qdata .= $_; }
 close(FILE);
 $nqdata = &edit_grace_file($qdata, $fs, $bt, $bu, $ft, $fu);
-open(FILE, "> $f");
-print FILE $nqdata;
-close(FILE);
+&open_tempfile(FILE, "> $f");
+&print_tempfile(FILE, $nqdata);
+&close_tempfile(FILE);
 
