@@ -22,7 +22,7 @@ $has_groups = ( $samba_version >= 3 && ($has_smbgroupedit || $has_net) );
 sub list_shares
 {
 local(@rv, $_);
-open(SAMBA, $config{smb_conf});
+&open_readfile(SAMBA, $config{smb_conf});
 while(<SAMBA>) {
 	chop; s/;.*$//g; s/^\s*#.*$//g;
 	if (/^\s*\[([^\]]+)\]/) {
@@ -41,7 +41,7 @@ sub get_share
 local($found, $_, $first, $arr);
 $arr = (@_==2 ? $_[1] : "share");
 undef(%$arr);
-open(SAMBA, $config{smb_conf});
+&open_readfile(SAMBA, $config{smb_conf});
 while(<SAMBA>) {
 	chop; s/^\s*;.*$//g; s/^\s*#.*$//g;
 	if (/^\s*\[([^\]]+)\]/) {
@@ -74,13 +74,13 @@ return $found;
 # Add an entry to the config file
 sub create_share
 {
-open(CONF, ">> $config{smb_conf}");
-print CONF "\n";
-print CONF "[$_[0]]\n";
+&open_tempfile(CONF, ">> $config{smb_conf}");
+&print_tempfile(CONF, "\n");
+&print_tempfile(CONF, "[$_[0]]\n");
 foreach $k (grep {!/share_name/} (keys %share)) {
-	print CONF "\t$k = $share{$k}\n";
+	&print_tempfile(CONF, "\t$k = $share{$k}\n");
 	}
-close(CONF);
+&close_tempfile(CONF);
 }
 
 
@@ -89,37 +89,39 @@ close(CONF);
 sub modify_share
 {
 local($_, @conf, $replacing, $first);
-open(CONF, $config{smb_conf});
+&open_readfile(CONF, $config{smb_conf});
 @conf = <CONF>;
 close(CONF);
-open(CONF, "> $config{smb_conf}");
+&open_tempfile(CONF, ">$config{smb_conf}");
 for($i=0; $i<@conf; $i++) {
 	chop($_ = $conf[$i]); s/;.*$//g; s/#.*$//g;
 	if (/^\s*\[([^\]]+)\]/) {
 		$first = 1;
 		if ($replacing) { $replacing = 0; }
 		elsif ($1 eq $_[0]) {
-			print CONF "[$_[1]]\n";
+			&print_tempfile(CONF, "[$_[1]]\n");
 			foreach $k (grep {!/share_name/} (keys %share)) {
-				print CONF "\t$k = $share{$k}\n";
+				&print_tempfile(CONF, "\t$k = $share{$k}\n");
 				}
-			print CONF "\n";
+			&print_tempfile(CONF, "\n");
 			$replacing = 1;
 			}
 		}
 	elsif (!$first && /^\s*([^=]*\S)\s*=\s*(.*)$/ && $_[0] eq "global") {
 		# found start of directives outside any share - assume [global]
 		$first = 1;
-		print CONF "[$_[1]]\n";
+		&print_tempfile(CONF, "[$_[1]]\n");
 		foreach $k (grep {!/share_name/} (keys %share)) {
-			print CONF "\t$k = $share{$k}\n";
+			&print_tempfile(CONF, "\t$k = $share{$k}\n");
 			}
-		print CONF "\n";
+		&print_tempfile(CONF, "\n");
 		$replacing = 1;
 		}
-	if (!$replacing) { print CONF $conf[$i]; }
+	if (!$replacing) {
+		&print_tempfile(CONF, $conf[$i]);
+		}
 	}
-close(CONF);
+&close_tempfile(CONF);
 }
 
 
@@ -128,22 +130,24 @@ close(CONF);
 sub delete_share
 {
 local($_, @conf, $deleting);
-open(CONF, $config{smb_conf});
+&open_readfile(CONF, $config{smb_conf});
 @conf = <CONF>;
 close(CONF);
-open(CONF, "> $config{smb_conf}");
+&open_tempfile(CONF, "> $config{smb_conf}");
 for($i=0; $i<@conf; $i++) {
 	chop($_ = $conf[$i]); s/;.*$//g;
 	if (/^\s*\[([^\]]+)\]/) {
 		if ($deleting) { $deleting = 0; }
 		elsif ($1 eq $_[0]) {
-			print CONF "\n";
+			&print_tempfile(CONF, "\n");
 			$deleting = 1;
 			}
 		}
-	if (!$deleting) { print CONF $conf[$i]; }
+	if (!$deleting) {
+		&print_tempfile(CONF, $conf[$i]);
+		}
 	}
-close(CONF);
+&close_tempfile(CONF);
 }
 
 
@@ -157,7 +161,14 @@ local($l, $started, @rv);
 if ($samba_version >= 3) {
 	# New samba status format
 	local %pidmap;
-	foreach $l (split(/\n/ , `$config{samba_status_program}`)) {
+	local $out;
+	local $ex = &execute_command("$config{samba_status_program} -s $config{smb_conf}", undef, \$out, undef);
+	if ($ex) {
+		# -s option not supported
+		&execute_command("$config{samba_status_program}",
+				 undef, \$out, undef);
+		}
+	foreach $l (split(/\n/ , $out)) {
 		if ($l =~ /^----/) {
 			$started++;
 			}
@@ -181,7 +192,10 @@ if ($samba_version >= 3) {
 	}
 else {
 	# Old samba status format
-	foreach $l (split(/\n/ , `$config{samba_status_program} -S`)) {
+	local $out;
+	&execute_command("$config{samba_status_program} -S",
+			 undef, \$out, undef);
+	foreach $l (split(/\n/, $out)) {
 		if ($l =~ /^----/) { $started++; }
 		if ($started && $l =~ /^(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.*)\s+\(\S+\)\s+(.*)$/ && (!$_[0] || $1 eq $_[0] || $1 eq $2 && $_[0] eq "homes")) {
 			push(@rv, [ $1, $2, $3, $4, $5, $6 ]);
@@ -197,7 +211,9 @@ return @rv;
 sub list_locks
 {
 local($l, $started, @rv);
-foreach $l (split(/\n/ , `$config{samba_status_program} -L`)) {
+local $out;
+&execute_command("$config{samba_status_program} -L", undef, \$out, undef);
+foreach $l (split(/\n/, $out)) {
 	if ($l =~ /^----/) { $started = 1; }
 	if ($started && $l =~ /^(\d+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.*)\s+(\S+\s+\S+\s+\d+\s+\d+:\d+:\d+\s+\d+)/) {
 		push(@rv, [ $1, $2, $3, $4, $5, $6 ]);
@@ -503,11 +519,11 @@ local(@rv, @b, $_, $lnum);
 if ($has_pdbedit) {
 	# Get list of users from the pdbedit command, which uses a configurable
 	# back-end for storage
-	open(PASS, "$config{'pdbedit'} -L -w -s $config{'smb_conf'} |");
+	&open_execute_command(PASS, "cd / && $config{'pdbedit'} -L -w -s $config{'smb_conf'}", 1);
 	}
 else {
 	# Read the password file directly
-	open(PASS, $config{'smb_passwd'});
+	&open_readfile(PASS, $config{'smb_passwd'});
 	}
 while(<PASS>) {
 	$lnum++;
@@ -544,16 +560,17 @@ if ($has_pdbedit) {
 	local $ws = &indexof("W", @{$_[0]->{'opts'}}) >= 0 ? "-m" : "";
 	local @opts = grep { $_ ne "U" && $_ ne "W" } @{$_[0]->{'opts'}};
 	local $out = &backquote_logged(
-		"$config{'pdbedit'} -a -s $config{'smb_conf'} -u ".
+		"cd / && $config{'pdbedit'} -a -s $config{'smb_conf'} -u ".
 		quotemeta($_[0]->{'name'}).
+		($config{'sync_gid'} ? " -G $config{'sync_gid'}" : "").
 		" -c '[".join("", @opts)."]' $ws");
 	$? && &error("$config{'pdbedit'} failed : <pre>$out</pre>");
 	}
 else {
 	# Add direct to Samba password file
-	open(PASS, ">>$config{'smb_passwd'}");
-	print PASS &user_string($_[0]);
-	close(PASS);
+	&open_tempfile(PASS, ">>$config{'smb_passwd'}");
+	&print_tempfile(PASS, &user_string($_[0]));
+	&close_tempfile(PASS);
 	chown(0, 0, $config{'smb_passwd'});
 	chmod(0600, $config{'smb_passwd'});
 	}
@@ -568,7 +585,7 @@ if ($has_pdbedit) {
 	local @opts = grep { $_ ne "U" } @{$_[0]->{'opts'}};
 	&indexof("W", @{$_[0]->{'opts'}}) >= 0 && &error($text{'saveuser_ews'});
 	$out = &backquote_logged(
-		"$config{'pdbedit'} -r -s $config{'smb_conf'} -u ".
+		"cd / && $config{'pdbedit'} -r -s $config{'smb_conf'} -u ".
 		quotemeta($_[0]->{'name'}).
 		" -c '[".join("", @opts)."]' 2>&1");
 	$? && &error("$config{'pdbedit'} failed : <pre>$out</pre>");
@@ -586,7 +603,7 @@ sub delete_user
 if ($has_pdbedit) {
 	# Use the pdbedit command
 	local $out = &backquote_logged(
-		"$config{'pdbedit'} -x -s $config{'smb_conf'} -u ".
+		"cd / && $config{'pdbedit'} -x -s $config{'smb_conf'} -u ".
 		quotemeta($_[0]->{'name'}));
 	$? && &error("$config{'pdbedit'} failed : <pre>$out</pre>");
 	}
@@ -615,17 +632,22 @@ return join(":", @u)."\n";
 # Changes the password of a user in the encrypted password file
 sub set_password
 {
+local $qu = quotemeta($_[0]);
+local $qp = quotemeta($_[1]);
 if ($samba_version >= 2) {
-	local $temp = &tempname();
-	open(TEMP, ">$temp");
-	print TEMP $_[1],"\n",$_[1],"\n";
-	close(TEMP);
-	local $rv = system("$config{'samba_password_program'} -s \"$_[0]\" >/dev/null 2>&1 <$temp");
+	local $passin = "$_[1]\n$_[1]\n";
+	local $ex = &execute_command("$config{'samba_password_program'} -c $config{'smb_conf'} -s $qu", \$passin, undef, undef);
+	if ($ex) {
+		# Try without -c 
+		$rv = &execute_command("$config{'samba_password_program'} -s $qu", \$passin, undef, undef);
+		}
 	unlink($temp);
 	return !$rv;
 	}
 else {
-	local $out = `$config{'samba_password_program'} "$_[0]" "$_[1]" 2>&1 </dev/null`;
+	local $out;
+	&execute_command("$config{'samba_password_program'} $qu $qp",
+			 undef, \$out, \$out);
 	return $out =~ /changed/i;
 	}
 }
@@ -723,7 +745,7 @@ if ($has_smbgroupedit) {
 else {
 	$cmd = "$config{'net'} -s $config{'smb_conf'} groupmap list verbose";
 	}
-open(GROUPS, "$cmd |");
+&open_execute_command(GROUPS, $cmd, 1);
 while(<GROUPS>) {
 	s/\r|\n//g;
 	if (/^(\S.*)/) {
@@ -836,11 +858,15 @@ else {
 # get_samba_version(&out)
 sub get_samba_version
 {
-if ((${$_[0]} = `$config{'samba_server'} -V 2>&1`) =~ /(Version|Samba)\s+(CVS\s+)?[^0-9 ]*(\d+)\.(\S+)/i ||
-    (${$_[0]} = `$config{'samba_server'} -v 2>&1`) =~ /(Version|Samba)\s+(CVS\s+)?[^0-9 ]*(\d+)\.(\S+)/i) {
-	$v1 = $3; $v2 = $4;
-	$v2 =~ s/[^0-9]//g;
-	return "$v1.$v2";
+local $flag;
+foreach $flag ("-V", "-v") {
+	&execute_command("$config{'samba_server'} $flag", undef, $_[0], $_[0]);
+	if (${$_[0]} =~ /(Version|Samba)\s+(CVS\s+)?[^0-9 ]*(\d+)\.(\S+)/i) {
+		local $v1 = $3;
+		local $v2 = $4;
+		$v2 =~ s/[^0-9]//g;
+		return "$v1.$v2";
+		}
 	}
 return undef;
 }
