@@ -444,7 +444,7 @@ sub installed_file
 sub delete_package
 {
         local(%fileset, $file, $out, $rv);
-	local $temp = &tempname();
+	local $temp = &transname();
 
 	$file = `lslpp -L -c $_[0] 2>&1 | grep -v '#'`;
 	%fileset = &fileset_info($file);
