@@ -267,7 +267,7 @@ elsif ($file =~ /\*|\?/) {
 		}
 	&error(&text('rpm_erpm', $_[1], $out)) if ($file eq $_[0]);
 	}
-local $temp = &tempname();
+local $temp = &transname();
 local $rv;
 if ($file =~ /\/[^\s]+[_\-]src[_\-]\d[^\s]*\.tar\.bz2$/) {
     $rv = install_cygwin_src_pkg($file, $temp, $in->{'root'});
@@ -449,7 +449,7 @@ sub delete_package
 	my $run_scripts = 1;
 	$run_scripts = 0 if (defined($in->{'noscripts'}) &&
 			     $in->{'noscripts'} == 1);
-	local $temp = &tempname();
+	local $temp = &transname();
 	my $rv = remove_cygwin_pkg($_[0], $temp, $root, $run_scripts);
 	if ($rv && open(FILE, "<$temp")) {
 	    my $out = join('', <FILE>);
