@@ -169,7 +169,7 @@ else {
 	local $cmd;
 	foreach $cmd ('gunzip', 'bunzip2') {
 		next if (!&has_command($cmd));
-		local $temp = &tempname();
+		local $temp = &transname();
 		mkdir($temp, 0700);
 		local $rv = system("cd $temp ; $cmd -c $_[0] | tar xf - +CONTENTS +COMMENT >/dev/null 2>&1");
 		if ($rv) {
