@@ -5,18 +5,19 @@
 # Returns HTML for entering a package to install
 sub update_system_input
 {
-return "$text{'yum_input'} <input name=update size=20> <input type=button onClick='window.ifield = document.forms[2].update; chooser = window.open(\"yum_find.cgi\", \"chooser\", \"toolbar=no,menubar=no,scrollbars=yes,width=600,height=500\")' value=\"$text{'yum_find'}\">";
+return "$text{'yum_input'} <input name=update size=20> <input type=button onClick='window.ifield = form.update; chooser = window.open(\"../$module_name/yum_find.cgi\", \"chooser\", \"toolbar=no,menubar=no,scrollbars=yes,width=600,height=500\")' value=\"$text{'yum_find'}\">";
 }
 
-# update_system_install()
+# update_system_install([package])
 # Install some package with yum
 sub update_system_install
 {
+local $update = $_[0] || $in{'update'};
 local (@rv, @newpacks);
-print "<b>",&text('yum_install', "<tt>yum -y install $in{'update'}</tt>"),"</b><p>\n";
+print "<b>",&text('yum_install', "<tt>yum -y install $update</tt>"),"</b><p>\n";
 print "<pre>";
-&additional_log('exec', undef, "yum -y install $in{'update'}");
-open(CMD, "yum -y install $in{'update'} 2>&1 </dev/null |");
+&additional_log('exec', undef, "yum -y install $update");
+open(CMD, "yum -y install $update 2>&1 </dev/null |");
 while(<CMD>) {
 	if (/^\[(update|install|deps):\s+(\S+)\s+/) {
 		push(@rv, $2);
