@@ -1,5 +1,9 @@
 
 require 'squid-lib.pl';
+@accopts = ('portsnets', 'othercaches', 'musage', 'logging', 'copts',
+	    'hprogs', 'actrl', 'admopts', 'proxyauth', 'miscopt', 'cms',
+	    'rebuild', 'calamaris', 'delay', 'headeracc', 'refresh',
+	    'authparam', 'iptables');
 
 # acl_security_form(&options)
 # Output HTML for editing security options for the squid module
@@ -7,9 +11,7 @@ sub acl_security_form
 {
 print "<tr> <td valign=top><b>$text{'acl_sections'}</b></td>\n";
 print "<td colspan=3><select name=sections multiple size=6>\n";
-foreach $s ('portsnets', 'othercaches', 'musage', 'logging', 'copts',
-	    'hprogs', 'actrl', 'admopts', 'proxyauth', 'miscopt', 'cms',
-	    'rebuild', 'calamaris', 'delay', 'headeracc', 'authparam') {
+foreach $s (@accopts) {
 	printf "<option value=%s %s>%s\n",
 		$s, $_[0]->{$s} ? 'selected' : '', $text{"index_${s}"};
 	}
@@ -24,6 +26,12 @@ printf "<td><input type=radio name=start value=1 %s> %s\n",
 	$_[0]->{'start'} ? 'checked' : '', $text{'yes'};
 printf "<input type=radio name=start value=0 %s> %s</td> </tr>\n",
 	$_[0]->{'start'} ? '' : 'checked', $text{'no'};
+
+print "<tr> <td><b>$text{'acl_restart'}</b></td>\n";
+printf "<td><input type=radio name=restart value=1 %s> %s\n",
+	$_[0]->{'restart'} ? 'checked' : '', $text{'yes'};
+printf "<input type=radio name=restart value=0 %s> %s</td> </tr>\n",
+	$_[0]->{'restart'} ? '' : 'checked', $text{'no'};
 }
 
 # acl_security_save(&options)
@@ -32,11 +40,10 @@ sub acl_security_save
 {
 $_[0]->{'root'} = $in{'root'};
 map { $sections{$_} = 1 } split(/\0/, $in{'sections'});
-foreach $s ('portsnets', 'othercaches', 'musage', 'logging', 'copts',
-	    'hprogs', 'actrl', 'admopts', 'proxyauth', 'miscopt', 'cms',
-	    'rebuild', 'calamaris', 'delay', 'authparam') {
+foreach $s (@accopts) {
 	$_[0]->{$s} = $sections{$s};
 	}
 $_[0]->{'start'} = $in{'start'};
+$_[0]->{'restart'} = $in{'restart'};
 }
 
