@@ -60,14 +60,17 @@ foreach $c (@{$_[1]}) {
 return wantarray ? @rv : $rv[0];
 }
 
-# save_directive(name, &config, [value*])
+# save_directive(name, &config, [value*|&values], [before])
 sub save_directive
 {
 local @o = &find($_[0], $_[1]);
-local @n = grep { defined($_) } @_[2..@_-1];
+local @n = ref($_[2]) ?
+		grep { defined($_) } @{$_[2]} :
+		grep { defined($_) } @_[2..@_-1];
 local $lref = &read_file_lines($_[1]->[0]->{'file'});
 local $id = ("\t" x $_[1]->[0]->{'indent'});
 local $i;
+local $before = $_[3] && ref($_[2]) ? &find($_[3], $_[1]) : undef;
 for($i=0; $i<@o || $i<@n; $i++) {
 	if ($o[$i] && $n[$i]) {
 		# Replacing a line
@@ -82,11 +85,20 @@ for($i=0; $i<@o || $i<@n; $i++) {
 				}
 			}
 		}
-	elsif ($n[$i]) {
-		# Adding a line
+	elsif ($n[$i] && !$before) {
+		# Adding a line at the end
 		local $ll = $_[1]->[@{$_[1]}-1]->{'line'};
 		splice(@$lref, $ll+1, 0, "$id$_[0] $n[$i]");
 		}
+	elsif ($n[$i] && $before) {
+		# Adding a line before the first instance of some directive
+		splice(@$lref, $before->{'line'}, 0, "$id$_[0] $n[$i]");
+		foreach $c (@{$_[1]}) {
+			if ($c->{'line'} >= $before->{'line'}) {
+				$c->{'line'}--;
+				}
+			}
+		}
 	}
 }
 
