@@ -8,10 +8,10 @@ sub get_apache_status
 return { 'up' => -1 } if (!&foreign_check($_[1]));
 local %aconfig = &foreign_config($_[1]);
 return { 'up' => -1 } if (!-x $aconfig{'httpd_path'});
-local $file = &foreign_call($_[1], "get_pid_file");
-if (open(PID, $file) && <PID> =~ /(\d+)/ && kill(0, $1)) {
-	close(PID);
-	local @st = stat($file);
+
+local $pidfile = &foreign_call($_[1], "get_pid_file");
+if (&check_pid_file($pidfile)) {
+	local @st = stat($pidfile);
 	return { 'up' => 1,
 		 'desc' => &text('up_since', scalar(localtime($st[9]))) };
 	}
