@@ -20,8 +20,10 @@ sub show_iface_dialog
 &foreign_require("net", "net-lib.pl");
 print "<tr> <td><b>$text{'iface_iface'}</b></td>\n";
 print "<td><select name=iface>\n";
+local ($i, %done);
 foreach $i (sort { $a->{'fullname'} cmp $b->{'fullname'} }
-		 &net::boot_interfaces()) {
+		 (&net::boot_interfaces(), &net::active_interfaces())) {
+	next if ($done{$a->{'fullname'}}++);
 	printf "<option value=%s %s>%s (%s)\n",
 		$i->{'fullname'},
 		$_[0]->{'iface'} eq $i->{'fullname'} ? 'selected' : '',
