@@ -23,8 +23,9 @@ require './status-lib.pl';
 
 # Check for services that are down
 $now = localtime(time());
-$host = &get_system_hostname();
+$thishost = &get_system_hostname();
 @services = &list_services();
+$ecount = 0;
 foreach $serv (@services) {
 	if ($serv->{'nosched'} == 1) {
 		# Scheduled checking totally disabled
@@ -52,6 +53,9 @@ foreach $serv (@services) {
 		$fails{$serv->{'id'}} = 0;
 		}
 
+	# Work out the hostname
+	local $host = $serv->{'remote'} || $thishost;
+
 	if ($warn == 0 && $up == 0 && $o) {
 		# Service has just gone down
 		$email .= "Monitor on $host for '$serv->{'desc'}'\n".
@@ -64,6 +68,7 @@ foreach $serv (@services) {
 			$email .= $out;
 			}
 		$email .= "\n";
+		$ecount++;
 		}
 	elsif ($warn == 1 && $up != $o &&
 	       (defined($o) || $up == 0)) {
@@ -79,6 +84,7 @@ foreach $serv (@services) {
 				$email .= $out;
 				}
 			$email .= "\n";
+			$ecount++;
 			}
 		elsif ($up == 1) {
 			$email .= "Monitor on $host for '$serv->{'desc'}'\n".
@@ -91,6 +97,7 @@ foreach $serv (@services) {
 				$email .= $out;
 				}
 			$email .= "\n";
+			$ecount++;
 			}
 		elsif ($up == -1) {
 			$email .= "Monitor on $host for '$serv->{'desc'}'\n".
@@ -98,6 +105,7 @@ foreach $serv (@services) {
 			$subj = "$serv->{'desc'} uninstalled on $host";
 		        $pager_msg .= "$host: \"$serv->{'desc'}\" uninstalled $now! ";
 			push(@snmp_msg, "$host: $serv->{'desc'} uninstalled");
+			$ecount++;
 			}
 		elsif ($up == -2) {
 			$email .= "Monitor on $host for '$serv->{'desc'}'\n".
@@ -105,6 +113,7 @@ foreach $serv (@services) {
 			$subj = "$serv->{'desc'} Webmin down on $host";
 		        $pager_msg .= "$host: \"$serv->{'desc'}\" Webmin down $now! ";
 			push(@snmp_msg, "$host: $serv->{'desc'} Webmin down");
+			$ecount++;
 			}
 		elsif ($up == -3) {
 			$email .= "Monitor on $host for '$serv->{'desc'}'\n".
@@ -112,6 +121,7 @@ foreach $serv (@services) {
 			$subj = "$serv->{'desc'} timed out on $host";
 		        $pager_msg .= "$host: \"$serv->{'desc'}\" timed out $now! ";
 			push(@snmp_msg, "$host: $serv->{'desc'} timed out");
+			$ecount++;
 			}
 		}
 	elsif ($warn == 2 && $up == 0) {
@@ -126,11 +136,13 @@ foreach $serv (@services) {
 			$email .= $out;
 			}
 		$email .= "\n";
+		$ecount++;
 		}
 	$oldstatus{$serv->{'id'}} = $up;
 	if ($config{'sched_single'} && $email) {
 		# Force the sending of one email (and SNMP trap) per report
-		&send_status_email($email, $subj);
+		&send_status_email($email,
+		  $config{'subject_mode'} ? $subj : "Service monitor : $subj");
 		undef($email);
 		}
 
@@ -147,9 +159,12 @@ foreach $serv (@services) {
 &write_file($fails_file, \%fails);
 &unlock_file($fails_file);
 
-# Send the email if necessary
-if ($email) {
-	&send_status_email($email, 'Service monitor');
+# Send the email with all messages, if necessary
+if ($ecount && !$config{'sched_single'}) {
+	&send_status_email($email,
+			   $config{'subject_mode'} ? "Service monitor" :
+			   $ecount == 1 ? "Service monitor : $subj" :
+			   	          "Service monitor : $ecount services");
 	&send_status_pager($pager_msg);
 	}
 
