@@ -37,6 +37,7 @@ sub show_raid_dialog
 local $conf = &raid::get_raidtab();
 print "<tr> <td><b>$text{'raid_device'}</b></td>\n";
 print "<td><select name=device>\n";
+local $found = !$_[0]->{'device'};
 foreach $c (@$conf) {
 	local $lvl = &raid::find_value('raid-level', $c->{'members'});
 	printf "<option value=%s %s>%s - %s\n",
@@ -45,14 +46,20 @@ foreach $c (@$conf) {
 		$c->{'value'},
 		$lvl eq 'linear' ? $raid::text{'linear'}
 				 : $raid::text{"raid$lvl"};
+	$found++ if ($_[0]->{'device'} eq $c->{'value'});
 	}
-print "</select></td> </tr>\n";
+printf "<option value='' %s>%s\n",
+	$found ? "" : "selected", $text{'raid_other'};
+print "</select>\n";
+printf "<input name=other size=10 value='%s'>\n",
+	$found ? "" : $_[0]->{'device'};
+print "</td> </tr>\n";
 }
 
 sub parse_raid_dialog
 {
 &depends_check($_[0], "raid");
-$in{'device'} || &error($text{'raid_edevice'});
-$_[0]->{'device'} = $in{'device'};
+$_[0]->{'device'} = $in{'device'} || $in{'other'};
+$_[0]->{'device'} =~ /^\S+$/ || &error($text{'raid_edevice'});
 }
 
