@@ -8,10 +8,10 @@ $lconf = &get_log_config($ARGV[0]);
 $lconf || die "Logfile $ARGV[0] does not exist";
 
 open(NULL, ">/dev/null");
-&generate_report($ARGV[0], NULL, 0);
+$ok = &generate_report($ARGV[0], NULL, 0);
 close(NULL);
 
-if ($lconf->{'clear'}) {
+if ($ok && $lconf->{'clear'}) {
 	# Truncate or delete the files for this report
 	foreach $f (&all_log_files($ARGV[0])) {
 		next if (!-r $f);
