@@ -99,9 +99,9 @@ else {
 # save_connect_details(ip, pid, section)
 sub save_connect_details
 {
-open(DETAILS, ">$details_file");
-print DETAILS $_[0],"\n",$_[1],"\n",$_[2],"\n";
-close(DETAILS);
+&open_tempfile(DETAILS, ">$details_file");
+&print_tempfile(DETAILS, $_[0],"\n",$_[1],"\n",$_[2],"\n");
+&close_tempfile(DETAILS);
 }
 
 # get_connect_details()
@@ -181,7 +181,7 @@ if ($_[1] == 1) {
 	print &text('connect_cmd', "$config{'wvdial'} $sect"),"\n\n";
 	}
 elsif ($_[1] == 0) {
-	print "<p><b>",&text('connect_cmd',
+	print "<b>",&text('connect_cmd',
 			     "<tt>$config{'wvdial'} $sect</tt>"),"</b><br>\n";
 	print "<pre>";
 	}
